/* MCVE v3.x C API
   (c) 2002 by Main Street Softworks, Inc.
   Written by Brad House

   This API is being released to the public domain to be modified and used in any manor the holder of this code sees fit.
   For any questions please contact  support@mainstreetsoftworks.com
*/

#include "libmcve_main.h"
#include "mcve.h"


#ifdef ENABLE_SSL
SSL_CTX *client_ctx;
#endif


/* BASE ROUTINES */

int MCVE_InitEngine(char *location)
{
  char egd[255];
  egd[0]=0;
#ifndef DISABLE_IP
  if (!MCVE_InitSockets()) return(0);
#endif
#ifdef ENABLE_SSL
  SSL_load_error_strings();
  OpenSSL_add_ssl_algorithms();
#ifdef NEED_SEED
  strcpy(egd, "/var/run/egd-pool");
  if (RAND_egd(egd) == -1)
    return(0);
#endif
  client_ctx=SSL_CTX_new(SSLv23_client_method());
  if (!SSL_CTX_load_verify_locations(client_ctx, location, NULL)) ;
  if (client_ctx == NULL) return(0);
#endif
 return(1);
}

void MCVE_DestroyEngine()
{
#ifdef ENABLE_SSL
  SSL_CTX_free(client_ctx);
#endif
}

//Initializes the MCVE Connection structure
void MCVE_InitConn(MCVE_CONN *myconn)
{
  _MCVE_CONN *conn;
  myconn[0]=(MCVE_CONN)malloc(sizeof(_MCVE_CONN)*1);
  conn=(_MCVE_CONN *)myconn[0];

  conn->conn_method=-1;
  conn->port=-1;
  conn->ptr=-1;
  conn->timeout=-1;
  conn->blocking=0;
  conn->status=0;
  conn->verifyconn=1;
  conn->verifyssl=0;
  conn->max_conn_time=5;
  conn->inbuf=NULL;
  conn->outbuf=NULL;
  conn->error_text=NULL;
#ifdef ENABLE_SSL
  conn->ssl=NULL;
  conn->server_cert=NULL;
#endif
#ifdef MCVE_THREADSAFE
  MCVE_init_locks(myconn);
#endif
  conn->outstanding_auths=0;
  conn->queue_length=0;
  conn->queue=NULL;
  memset(conn->location, 0, 250);
}

int MCVE_SetBlocking(MCVE_CONN *myconn, int tf)
{
  _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
  conn->blocking=tf;
  return(1);
}

int MCVE_SetTimeout(MCVE_CONN *myconn, long timeout)
{
  _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
  conn->timeout=timeout;
  return(1);
}

// Sets method and path for drop files
// Returns 0 on error
int MCVE_SetDropFile(MCVE_CONN *myconn, char *df_location)
{
  _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
  if (strlen(df_location) > 249) return(0);

  if (!MCVE_DirectoryExists(df_location))
    return(0);

  strncpy(conn->location, df_location,250);
  conn->conn_method=MCVE_FILE;
  conn->status=MCVE_CONNECTED;
  return(1);
}



// Sets method,location, and portnum for IP
// Returns 0 on error
int MCVE_SetIP(MCVE_CONN *myconn, char *host, unsigned long port)
{
#ifndef DISABLE_IP  
  _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
  if (strlen(host) > 249 || strlen(host) < 1) return(0);
  strncpy(conn->location, host,250);
  if (port < 1) return(0);
  conn->port=port;
  conn->conn_method=MCVE_SOCKETS;
  return(1);
#else
  return(0);
#endif


}


int
MCVE_SetSSL_Files(const char *sslkeyfile, const char *sslcertfile)
{
  int retval;

  if (sslkeyfile != NULL) {
    retval = SSL_CTX_use_PrivateKey_file(client_ctx, sslkeyfile, SSL_FILETYPE_PEM);
    if (retval != 1)
      return (0);
  }

  if (sslcertfile != NULL) {
    retval = SSL_CTX_use_certificate_file(client_ctx, sslcertfile, SSL_FILETYPE_PEM);
    if (retval != 1)
      return (0);
  }

  return (1);
}

// Sets method and path for drop files
// Returns 0 on error
int MCVE_SetSSL(MCVE_CONN *myconn, char *host, unsigned long port)
{
  _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
#ifdef ENABLE_SSL
  if (host == NULL || strlen(host) > 249 || strlen(host) < 1) {
    MCVE_Set_Conn_Error(myconn, "Invalid Host Name");  // Set Error Text
    return(0);
  }
  strncpy(conn->location, host, 250);
  if (port < 1) {
    MCVE_Set_Conn_Error(myconn, "Invalid Port Number");  // Set Error Text
    return(0);
  }
  conn->port=port;
  conn->conn_method=MCVE_SSL;
  return(1);
#else
  return(0);
#endif
}


void MCVE_VerifyConnection(MCVE_CONN *myconn, int tf)
{
  _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
  conn->verifyconn=tf;
}

void MCVE_VerifySSLCert(MCVE_CONN *myconn, int tf)
{
  _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
  conn->verifyssl=tf;
}

int MCVE_Connect(MCVE_CONN *myconn)
{
  _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
#ifdef ENABLE_SSL
  long err;
#endif
  if (conn->conn_method == MCVE_FILE) {
    return(1);  // No connection necessary for Drop File
#ifndef DISABLE_IP
  } else if (conn->conn_method == MCVE_SOCKETS) {
    conn->ptr=MCVE_ip_connect(myconn);
    if (conn->ptr == -1) {
       return(0);  //Connection Failed
    } 
#ifdef ENABLE_SSL
  } else if (conn->conn_method == MCVE_SSL) {
    conn->ptr=MCVE_ip_connect(myconn);
     if (conn->ptr == -1) {
        return(0);  //Connection Failed
     }
     conn->ssl=SSL_new(client_ctx);
     SSL_set_fd(conn->ssl, conn->ptr);
     if (SSL_connect(conn->ssl) == -1) {
        MCVE_Set_Conn_Error(myconn, "SSL Negotiation Failed");  // Set Error Text
	SSL_free(conn->ssl);
	if (conn->ptr != -1)
          MCVE_CloseSocket(conn->ptr);
        conn->ptr=-1;
        return(0);
     }
     conn->server_cert = SSL_get_peer_certificate(conn->ssl);
     //printf("Server Certificate subject: %s\r\n", X509_NAME_oneline(X509_get_subject_name((X509 *)conn->server_cert),0,0));
    // printf("Server Certificate issuer: %s\r\n", X509_NAME_oneline(X509_get_issuer_name((X509 *)conn->server_cert),0,0));

     if (conn->verifyssl && (err=SSL_get_verify_result(conn->ssl)) != X509_V_OK) {  // SSL Certificate check failed
       MCVE_Set_Conn_Error(myconn, "SSL Certificate verification failed");
       if (conn->ptr != -1)
         MCVE_CloseSocket(conn->ptr);
       conn->ptr=-1;
       SSL_free((SSL *)conn->ssl);
       return(0);
     }
#endif /* ENABLE_SSL */
#endif /* DISABLE_IP */
  }
  conn->status = MCVE_CONNECTED;
#ifndef DISABLE_IP
  if ((conn->conn_method == MCVE_SOCKETS || conn->conn_method == MCVE_SSL) && conn->verifyconn) {
    if (!MCVE_VerifyPing(myconn)) {
      MCVE_Set_Conn_Error(myconn, "MCVE did not respond to a PING request. Ensure proper port number and MCVE v2.1 or greater.");
      if (conn->ptr != -1)
         MCVE_CloseSocket(conn->ptr);
       conn->ptr=-1;
#ifdef ENABLE_SSL
       if (conn->conn_method == MCVE_SSL)
         SSL_free(conn->ssl);
#endif
      conn->status=MCVE_DISCONNECTED;
      return(0);
     }
   }
#endif /* DISABLE_IP */
  conn->status=MCVE_CONNECTED;
  return(1);
}

void MCVE_MaxConnTimeout(MCVE_CONN *myconn, int maxtime)
{
  _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
  conn->max_conn_time=maxtime;
}

char *MCVE_ConnectionError(MCVE_CONN *myconn)
{
  _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
  return(conn->error_text);
}


// Closes MCVE Connection -- and is ready for MCVE_CONN to be used again
void MCVE_DestroyConn(MCVE_CONN *myconn)
{
  long i;
  _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];

  for (i=0; i<conn->queue_length; i++)
      MCVE_DeleteTrans(myconn, i);

#ifdef ENABLE_SSL
  if (conn->conn_method == MCVE_SSL)
    SSL_free((SSL *)conn->ssl);
#endif
#ifdef MCVE_THREADSAFE
  MCVE_destroy_locks(myconn);
#endif
  conn->conn_method=-1;
  conn->port=-1;
  if (conn->ptr != -1)
#ifndef DISABLE_IP
    MCVE_CloseSocket(conn->ptr);
#endif
  conn->ptr=-1;
  if (conn->inbuf != NULL)
    free(conn->inbuf);
  if (conn->outbuf != NULL)
    free(conn->outbuf);
  if (conn->error_text != NULL)
    free(conn->error_text);
  conn->error_text=NULL;
  conn->inbuf=NULL;
  conn->outbuf=NULL;
  conn->timeout=0;
  conn->status=0;
  conn->blocking=0;
  conn->verifyssl=0;
  conn->verifyconn=0;
  conn->max_conn_time=5;
  conn->outstanding_auths=0;
  conn->queue_length=0;
  if (conn->queue != NULL)
    free(conn->queue);
  conn->queue=NULL;

  free(conn);
  conn = NULL;
}



int MCVE_Monitor(MCVE_CONN *myconn)
{
  _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
  int ret=0;
  
  if (conn->status == MCVE_DISCONNECTED) {
    MCVE_Set_Conn_Error(myconn, "Not Connected to MCVE");  // Set Error Text
    return(0);
  }

  if (conn->conn_method == MCVE_FILE) {
    ret=MCVE_Monitor_File(myconn);
  } else if (conn->conn_method == MCVE_SOCKETS || conn->conn_method==MCVE_SSL) {
#ifndef DISABLE_IP
    ret=MCVE_Monitor_IP(myconn);

    if (!ret) {
      close(conn->ptr);
      conn->ptr=-1;
#ifdef ENABLE_SSL
      if (conn->conn_method == MCVE_SSL) SSL_free((SSL *)conn->ssl);
#endif /*ENABLE_SSL */
      MCVE_Set_Conn_Error(myconn, "Unexpected disconnection from MCVE");  // Set Error Text
      conn->status=MCVE_DISCONNECTED;
    }
    MCVE_ProcessBuffer(myconn);
#else 
    return(0);
#endif /* DISABLE_IP */
  }
 return(ret);
}

int MCVE_TransactionsSent(MCVE_CONN *myconn)
{
  _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
  int ret=0;
  if (conn->conn_method == MCVE_FILE) {
    ret=1;
  } else {
    MCVE_lock(myconn, MCVE_LOCK_OUT);
    if (conn->outbuf == NULL) ret=1;
    MCVE_unlock(myconn, MCVE_LOCK_OUT);
  }
  return(ret);
}



void MCVE_DeleteTrans(MCVE_CONN *myconn, long identifier)
{
  _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
  int i, j;
  MCVE_lock(myconn, MCVE_LOCK_QUEUE);
  if (conn->queue[identifier].transaction != NULL) {
    for (i=0; i<conn->queue[identifier].transaction_fields; i++) {
      free(conn->queue[identifier].transaction[i].key);
      free(conn->queue[identifier].transaction[i].value);
    }
    free(conn->queue[identifier].transaction);
    conn->queue[identifier].transaction=NULL;
  }
  conn->queue[identifier].transaction_fields=0;
  if (conn->queue[identifier].parsed_resp != NULL) {
    for (i=0; i<conn->queue[identifier].resp_fields; i++) {
      free(conn->queue[identifier].parsed_resp[i].key);
      free(conn->queue[identifier].parsed_resp[i].value);
    }
    free(conn->queue[identifier].parsed_resp);
    conn->queue[identifier].parsed_resp=NULL;
  }
  conn->queue[identifier].resp_fields=0;

  memset(conn->queue[identifier].identifier, 0, 29);
  conn->queue[identifier].status=MCVE_UNUSED;
  if (conn->queue[identifier].auth != NULL)
    free(conn->queue[identifier].auth);
  conn->queue[identifier].auth=NULL;
  if (conn->queue[identifier].text != NULL)
    free(conn->queue[identifier].text);
  conn->queue[identifier].text=NULL;
  if (conn->queue[identifier].item != NULL)
    free(conn->queue[identifier].item);
  conn->queue[identifier].item=NULL;
  if (conn->queue[identifier].batch != NULL)
    free(conn->queue[identifier].batch);
  conn->queue[identifier].batch=NULL;
  conn->queue[identifier].code=-1;
  conn->queue[identifier].avs=-1;
  conn->queue[identifier].cv=-1;
  conn->queue[identifier].tid=-1;
  if (conn->queue[identifier].response != NULL)
    free(conn->queue[identifier].response);
  conn->queue[identifier].response=NULL;
 // conn->queue[identifier].iscommadelimiated=0;
  if (conn->queue[identifier].separated != NULL) {
    for (i = 0; i < conn->queue[identifier].rows + 1; i++) {
      for (j = 0; j < conn->queue[identifier].columns; j++) {
        free(conn->queue[identifier].separated[i][j]);
      }
      free(conn->queue[identifier].separated[i]);
    }
    free(conn->queue[identifier].separated);
  }
  conn->queue[identifier].separated=NULL;
  conn->queue[identifier].columns=0;
  conn->queue[identifier].rows=0;
  conn->outstanding_auths--;
  MCVE_unlock(myconn, MCVE_LOCK_QUEUE);
}


/* Get a new Queue Number, return the location */
long MCVE_TransNew(MCVE_CONN *myconn)
{
  long pos;
  char timeout[20];
  _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];

  MCVE_lock(myconn, MCVE_LOCK_QUEUE);
  pos=MCVE_OpenQueueSlot(myconn);
  conn->queue[pos].status=MCVE_NEW;
  MCVE_ClearQueueSlot(myconn, pos);
  MCVE_unlock(myconn, MCVE_LOCK_QUEUE);

  if (conn->timeout > 0) {
    MCVE_snprintf(timeout, 20, "%ld", conn->timeout);
    MCVE_TransParam_Add(myconn, pos, "timeout", timeout);
  }
  return(pos);
}

int MCVE_VerifyTrans(MCVE_CONN *myconn, long identifier)
{
  return(1);
}

int MCVE_TransSend(MCVE_CONN *myconn, long identifier)
{
  _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
  int ret;

  if (!MCVE_VerifyTrans(myconn, identifier)) {
    return(0);
  }
 
  ret=MCVE_SendTransaction(myconn, identifier);

  if (ret) {
    conn->outstanding_auths++;

    /* Blocking mode is set, wait until trans is complete */
    if (conn->blocking) {
      while(MCVE_CheckStatus(myconn, identifier) == MCVE_PENDING) {
        if (!MCVE_Monitor(myconn)) {
	  return(0);
	}
	MCVE_uwait(50000);
      }
    }
  }
  return(ret);
}


char *MCVE_ResponseParam(MCVE_CONN *myconn, long identifier, char *key)
{
  _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
  int i;

  for (i=0; i<conn->queue[identifier].resp_fields; i++) {
    if (strcasecmp(key, conn->queue[identifier].parsed_resp[i].key) == 0)
      return(conn->queue[identifier].parsed_resp[i].value);
  }
  return(NULL);
}

/* Add a custom Transaction Parameter (modules, etc) */
int MCVE_TransParam_Add(MCVE_CONN *myconn, long identifier, char *key, char *value)
{
  _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
  int field;

  field=conn->queue[identifier].transaction_fields;

  conn->queue[identifier].transaction=(MCVE_TRANS *)realloc(conn->queue[identifier].transaction, (field+1)*sizeof(MCVE_TRANS));
  conn->queue[identifier].transaction[field].key=MC_SAFE_strdup(key);
  conn->queue[identifier].transaction[field].value=MC_SAFE_strdup(value);
  conn->queue[identifier].transaction_fields++;

  return(1);
}


/* Add a new Transaction Parameter */
int MCVE_TransParam(MCVE_CONN *myconn, long identifier, int key, ...)
{
  char key_string[100];
  char value_string[1024];
  int trans_type;
  int excharges_type;
  int temp_tf;
  char *temp_str=NULL;
  va_list ap;
  char *temp=NULL;
  _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];

  key_string[0]=0;
  value_string[0]=0;
  va_start(ap, key);

  switch(key) {
    case MC_TRANTYPE:
      MC_SAFE_strncpy(key_string, "action", 99);
      trans_type=va_arg(ap, int);
      if (trans_type < 1000) {
        MC_SAFE_strncpy(value_string, MCVE_GetTypeString(trans_type), 1023);
      } else {
        MC_SAFE_strncpy(value_string, MCVE_GetEngineAdminTypeString(trans_type), 1023);
      }
      conn->queue[identifier].type=trans_type;
    break;

    case MC_ADMIN:
      MC_SAFE_strncpy(key_string, "admin", 99);
      trans_type=va_arg(ap, int);
      MC_SAFE_strncpy(value_string, MCVE_GetAdminTypeString(trans_type), 1023);
      conn->queue[identifier].admin=trans_type;
    break;

    case MC_USERNAME:
      MC_SAFE_strncpy(key_string, "username", 99);
      MC_SAFE_strncpy(value_string, va_arg(ap, char *), 1023);
    break;

    case MC_PASSWORD:
      MC_SAFE_strncpy(key_string, "password", 99);
      MC_SAFE_strncpy(value_string, va_arg(ap, char *), 1023);
    break;

    case MC_ACCOUNT:
      MC_SAFE_strncpy(key_string, "account", 99);
      MC_SAFE_strncpy(value_string, va_arg(ap, char *), 1023);
    break;

    case MC_TRACKDATA:
      MC_SAFE_strncpy(key_string, "trackdata", 99);
      MC_SAFE_strncpy(value_string, va_arg(ap, char *), 1023);
    break;

    case MC_EXPDATE:
      MC_SAFE_strncpy(key_string, "expdate", 99);
      MC_SAFE_strncpy(value_string, va_arg(ap, char *), 1023);
    break;

    case MC_STREET:
      MC_SAFE_strncpy(key_string, "street", 99);
      MC_SAFE_strncpy(value_string, va_arg(ap, char *), 1023);
    break;

    case MC_ZIP:
      MC_SAFE_strncpy(key_string, "zip", 99);
      MC_SAFE_strncpy(value_string, va_arg(ap, char *), 1023);
    break;

    case MC_CV:
      MC_SAFE_strncpy(key_string, "cvv2", 99);
      MC_SAFE_strncpy(value_string, va_arg(ap, char *), 1023);
    break;

    case MC_COMMENTS:
      MC_SAFE_strncpy(key_string, "comments", 99);
      MC_SAFE_strncpy(value_string, va_arg(ap, char *), 1023);
    break;

    case MC_CLERKID:
      MC_SAFE_strncpy(key_string, "clerkid", 99);
      MC_SAFE_strncpy(value_string, va_arg(ap, char *), 1023);
    break;

    case MC_STATIONID:
      MC_SAFE_strncpy(key_string, "stationid", 99);
      MC_SAFE_strncpy(value_string, va_arg(ap, char *), 1023);
    break;

    case MC_APPRCODE:
      MC_SAFE_strncpy(key_string, "apprcode", 99);
      MC_SAFE_strncpy(value_string, va_arg(ap, char *), 1023);
    break;

    case MC_AMOUNT:
      MC_SAFE_strncpy(key_string, "amount", 99);
      MCVE_snprintf(value_string, 1023, "%.2f", va_arg(ap, double));
    break;

    case MC_PTRANNUM:
      MC_SAFE_strncpy(key_string, "ptrannum", 99);
      MCVE_snprintf(value_string, 1023, "%ld", va_arg(ap, long));
    break;

    case MC_TIMESTAMP:
      MC_SAFE_strncpy(key_string, "timestamp", 99);
      MCVE_snprintf(value_string, 1023, "%ld", va_arg(ap, long));
    break;

    case MC_TTID:
      MC_SAFE_strncpy(key_string, "ttid", 99);
      MCVE_snprintf(value_string, 1023, "%lld", va_arg(ap, MCVE_int64));
    break;

    case MC_USER:
      MC_SAFE_strncpy(key_string, "user", 99);
      MC_SAFE_strncpy(value_string, va_arg(ap, char *), 1023);
    break;

    case MC_PWD:
      MC_SAFE_strncpy(key_string, "pwd", 99);
      MC_SAFE_strncpy(value_string, va_arg(ap, char *), 1023);
    break;

    case MC_ACCT:
      MC_SAFE_strncpy(key_string, "acct", 99);
      MC_SAFE_strncpy(value_string, va_arg(ap, char *), 1023);
    break;

    case MC_BDATE:
      MC_SAFE_strncpy(key_string, "bdate", 99);
      MC_SAFE_strncpy(value_string, va_arg(ap, char *), 1023);
    break;

    case MC_EDATE:
      MC_SAFE_strncpy(key_string, "edate", 99);
      MC_SAFE_strncpy(value_string, va_arg(ap, char *), 1023);
    break;

    case MC_BATCH:
      MC_SAFE_strncpy(key_string, "batch", 99);
      MC_SAFE_strncpy(value_string, va_arg(ap, char *), 1023);
    break;

    case MC_FILE:
      MC_SAFE_strncpy(key_string, "file", 99);
      MC_SAFE_strncpy(value_string, va_arg(ap, char *), 1023);
    break;

    case MC_AUDITTYPE:
      MC_SAFE_strncpy(key_string, "type", 99);
      trans_type=va_arg(ap, int);
      MC_SAFE_strncpy(value_string, MCVE_GetTypeString(trans_type), 1023);
    break;

    case MC_CUSTOM:
      MC_SAFE_strncpy(key_string, va_arg(ap, char *), 99);
      MC_SAFE_strncpy(value_string, va_arg(ap, char *), 1023);
    break;

    /* Additional args for restaurant, lodging and auto rental */
    case MC_EXAMOUNT:
      MC_SAFE_strncpy(key_string, "examount", 99);
      MCVE_snprintf(value_string, 1023, "%.2f", va_arg(ap, double));
    break;
    
    case MC_EXCHARGES:
      MC_SAFE_strncpy(key_string, "excharges", 99);
      excharges_type = va_arg(ap, int);
      MC_SAFE_strncpy(value_string, MCVE_GetExchargesArgString(excharges_type), 1023);
    break;

    case MC_RATE:
      MC_SAFE_strncpy(key_string, "rate", 99);
      MCVE_snprintf(value_string, 1023, "%.2f", va_arg(ap, double));
    break;

    case MC_RENTERNAME:
      MC_SAFE_strncpy(key_string, "rentername", 99);
      MC_SAFE_strncpy(value_string, va_arg(ap, char *), 1023);
    break;    

    case MC_RETURNCITY:
      MC_SAFE_strncpy(key_string, "returncity", 99);
      MC_SAFE_strncpy(value_string, va_arg(ap, char *), 1023);
    break;    

    case MC_RETURNSTATE:
      MC_SAFE_strncpy(key_string, "returnstate", 99);
      MC_SAFE_strncpy(value_string, va_arg(ap, char *), 1023);
    break;    

    case MC_RETURNLOCATION:
      MC_SAFE_strncpy(key_string, "returnlocation", 99);
      MC_SAFE_strncpy(value_string, va_arg(ap, char *), 1023);
    break;   
    
    case MC_INQUIRY:
      MC_SAFE_strncpy(key_string, "inquiry", 99);
      temp_tf=va_arg(ap, int);
      if (temp_tf) {
        MC_SAFE_strncpy(value_string, "yes", 1023); 
      } else {
        MC_SAFE_strncpy(value_string, "no", 1023);
      }
    break;
    
    case MC_PRIORITY:
      MC_SAFE_strncpy(key_string, "priority", 99);
      MC_SAFE_strncpy(value_string, MCVE_GetPriorityString(va_arg(ap, int)), 1023);
    break;

    case MC_CARDTYPES:
      MC_SAFE_strncpy(key_string, "cardtypes", 99);
      MC_SAFE_strncpy(value_string, MCVE_GetCardTypesString(va_arg(ap, int)), 1023);
    break;

    case MC_SUB:
      MC_SAFE_strncpy(key_string, "sub", 99);
      MCVE_snprintf(value_string, 1023, "%d", va_arg(ap, int));
    break;

    case MC_MARKER:
      MC_SAFE_strncpy(key_string, "marker", 99);
      MCVE_snprintf(value_string, 1023, "%ld", va_arg(ap, long));
    break;

    case MC_DEVICETYPE:
      MC_SAFE_strncpy(key_string, "devicetype", 99);
      MC_SAFE_strncpy(value_string, va_arg(ap, char *), 1023);
    break;

    case MC_ERRORCODE:
      MC_SAFE_strncpy(key_string, "errorcode", 99);
      MC_SAFE_strncpy(value_string, va_arg(ap, char *), 1023);
    break;

    case MC_NEWBATCH:
      MC_SAFE_strncpy(key_string, "newbatch", 99);
      MC_SAFE_strncpy(value_string, va_arg(ap, char *), 1023);
    break;

    case MC_CURR:
      MC_SAFE_strncpy(key_string, "curr", 99);
      MC_SAFE_strncpy(value_string, va_arg(ap, char *), 1023);
    break;

    case MC_DESCMERCH:
      MC_SAFE_strncpy(key_string, "descmerch", 99);
      MC_SAFE_strncpy(value_string, va_arg(ap, char *), 1023);
    break;

    case MC_DESCLOC:
      MC_SAFE_strncpy(key_string, "descloc", 99);
      MC_SAFE_strncpy(value_string, va_arg(ap, char *), 1023);
    break;

    case MC_ORIGTYPE:
      MC_SAFE_strncpy(key_string, "origtype", 99);
      trans_type=va_arg(ap, int);
      if (trans_type < 1000) {
        MC_SAFE_strncpy(value_string, MCVE_GetTypeString(trans_type), 1023);
      } else {
        MC_SAFE_strncpy(value_string, MCVE_GetEngineAdminTypeString(trans_type), 1023);
      }
    break;

    case MC_VOIDORIGTYPE:
      MC_SAFE_strncpy(key_string, "voidorigtype", 99);
      trans_type=va_arg(ap, int);
      if (trans_type < 1000) {
        MC_SAFE_strncpy(value_string, MCVE_GetTypeString(trans_type), 1023);
      } else {
        MC_SAFE_strncpy(value_string, MCVE_GetEngineAdminTypeString(trans_type), 1023);
      }
    break;

    case MC_PIN:
      MC_SAFE_strncpy(key_string, "pin", 99);
      MC_SAFE_strncpy(value_string, va_arg(ap, char *), 1023);
    break;



    default:
      if (key >= 2000) {
        temp=MCVE_GetUserArgString(key);
	if (temp != NULL && strlen(temp)) {
          MC_SAFE_strncpy(key_string, temp, 99);
	  if (key == MC_USER_SUB) {
	    MCVE_snprintf(value_string, 1023, "%d", va_arg(ap, int));
	  } else if (key == MC_USER_CARDTYPES) {
	    temp_str=MCVE_GetCardTypesString(va_arg(ap, int));
	    MC_SAFE_strncpy(value_string, temp_str, 1023);
	    free(temp_str);
	  } else if (key == MC_USER_MODE) {
	    temp_str=MCVE_GetModeString(va_arg(ap, int));
	    MC_SAFE_strncpy(value_string, temp_str, 1023);
	    free(temp_str);
	  } else {
            MC_SAFE_strncpy(value_string, va_arg(ap, char *), 1023);
	  }
	}
      }
    break;
  };
  va_end(ap);
  if (strlen(key_string) && strlen(value_string)) {
    /* Maintain compatability with MCVE 2.0-2.1 */
    if (strcasecmp(key_string, "ttid") == 0) {
      MCVE_TransParam_Add(myconn, identifier, "sid", value_string);
    }
    return(MCVE_TransParam_Add(myconn, identifier, key_string, value_string));
  }
  return(0);
}


long MCVE_Ping(MCVE_CONN *myconn)
{
//  _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
  long identifier;
  identifier=MCVE_TransNew(myconn);
  MCVE_TransParam(myconn, identifier, MC_TRANTYPE, MC_TRAN_PING);
  if (!MCVE_TransSend(myconn, identifier))
    return(-1);
  return(identifier);
}


/* RESPONSE CHECKING ROUTINES */

int MCVE_ReturnStatus(MCVE_CONN *myconn, long identifier)
{
  _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
  int status;
  char *code;
  code=MCVE_ResponseParam(myconn, identifier, "code");
  if (code == NULL) return(MCVE_SUCCESS);
  MCVE_lock(myconn, MCVE_LOCK_QUEUE);
  if (conn->queue[identifier].code == -1)
    status=-1;
  else if (conn->queue[identifier].code == MCVE_SUCCESS)
    status=MCVE_SUCCESS;
  else if (conn->queue[identifier].code == MCVE_AUTH)
    status=MCVE_SUCCESS;
  else
    status=MCVE_FAIL;
  MCVE_unlock(myconn, MCVE_LOCK_QUEUE);
  return(status);
}

int MCVE_ReturnCode(MCVE_CONN *myconn, long identifier)
{
  _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
  int code;
  MCVE_lock(myconn, MCVE_LOCK_QUEUE);
    code=conn->queue[identifier].code;
  MCVE_unlock(myconn, MCVE_LOCK_QUEUE);
  return(code);
}

long MCVE_TransactionItem(MCVE_CONN *myconn, long identifier)
{
  _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
  long item=-1;
  MCVE_lock(myconn, MCVE_LOCK_QUEUE);
    if (conn->queue[identifier].item != NULL)
      item=atol(conn->queue[identifier].item);
  MCVE_unlock(myconn, MCVE_LOCK_QUEUE);
  return(item);
}

long MCVE_TransactionBatch(MCVE_CONN *myconn, long identifier)
{
  _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
  long batch=-1;
  MCVE_lock(myconn, MCVE_LOCK_QUEUE);
    if (conn->queue[identifier].batch != NULL)
      batch=atol(conn->queue[identifier].batch);
  MCVE_unlock(myconn, MCVE_LOCK_QUEUE);
  return(batch);
}

MCVE_int64 MCVE_TransactionID(MCVE_CONN *myconn, long identifier)
{
  _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
  MCVE_int64 tid=-1;
  MCVE_lock(myconn, MCVE_LOCK_QUEUE);
    if (conn->queue[identifier].tid != -1)
      tid=conn->queue[identifier].tid;
  MCVE_unlock(myconn, MCVE_LOCK_QUEUE);
  return(tid);
}

char *MCVE_TransactionAuth(MCVE_CONN *myconn, long identifier)
{
  _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
  char *auth=NULL;
  MCVE_lock(myconn, MCVE_LOCK_QUEUE);
    if (conn->queue[identifier].auth != NULL)
      auth=conn->queue[identifier].auth;
  MCVE_unlock(myconn, MCVE_LOCK_QUEUE);
  return(auth);
}

char *MCVE_TransactionText(MCVE_CONN *myconn, long identifier)
{
  _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
  char *text=NULL;
  MCVE_lock(myconn, MCVE_LOCK_QUEUE);
    if (conn->queue[identifier].text != NULL)
      text=conn->queue[identifier].text;
  MCVE_unlock(myconn, MCVE_LOCK_QUEUE);
  return(text);
}

int MCVE_TransactionAVS(MCVE_CONN *myconn, long identifier)
{
  _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
  int code;
  MCVE_lock(myconn, MCVE_LOCK_QUEUE);
    code=conn->queue[identifier].avs;
  MCVE_unlock(myconn, MCVE_LOCK_QUEUE);
  return(code);
}

int MCVE_TransactionCV(MCVE_CONN *myconn, long identifier)
{
  _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
  int code;
  MCVE_lock(myconn, MCVE_LOCK_QUEUE);
    code=conn->queue[identifier].cv;
  MCVE_unlock(myconn, MCVE_LOCK_QUEUE);
  return(code);
}

long MCVE_TransInQueue(MCVE_CONN *myconn)
{
  _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
  long auths;
  MCVE_lock(myconn, MCVE_LOCK_QUEUE);
  auths=conn->outstanding_auths;
  MCVE_unlock(myconn, MCVE_LOCK_QUEUE);
  return(auths);
}

int MCVE_CheckStatus(MCVE_CONN *myconn, long trans)
{
  _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
  int status;
  MCVE_lock(myconn, MCVE_LOCK_QUEUE);
  status=conn->queue[trans].status;
  MCVE_unlock(myconn, MCVE_LOCK_QUEUE);
  return(status);
}

long MCVE_CompleteAuthorizations(MCVE_CONN *myconn, long **listings)
{
  _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
  long num=0, i;
  MCVE_lock(myconn, MCVE_LOCK_QUEUE);
  for (i=0; i<conn->queue_length; i++) {
   if  (conn->queue[i].status == MCVE_DONE)
     num++;
  }
  if (num != 0) {
    listings[0] = (long *)malloc((num+1)*sizeof(long));
    num=0;
    for (i=0; i<conn->queue_length; i++) {
      if (conn->queue[i].status == MCVE_DONE) {
        listings[0][num]=i;
        num++;
      }
    }
  }
  MCVE_unlock(myconn, MCVE_LOCK_QUEUE);
  return(num);
}


char *MCVE_GetUserParam(MCVE_CONN *myconn, long identifier, int key)
{
  char *str_key=NULL;
  str_key=MCVE_GetUserArgString(key);
  if (str_key == NULL) return(NULL);
  return(MCVE_ResponseParam(myconn, identifier, str_key));
}


/* COMMA DELIMITED ROUTINES */

int MCVE_IsCommaDelimited(MCVE_CONN *myconn, long identifier)
{
  _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
  return(conn->queue[identifier].iscommadelimiated);
}


int MCVE_ParseCommaDelimited(MCVE_CONN *myconn, long identifier)
{
  _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
  int columns;
  long rows;
  int i;
  long j, row_num;
  int offset;
  char *temp, *temp2, *str, **real_row;
  char *line_temp1, *line_temp2;
  columns=MCVE_Count_CD_Columns(myconn, identifier);
  rows=MCVE_Count_CD_Lines(myconn, identifier);
  conn->queue[identifier].separated=(char ***)malloc(rows*sizeof(char **));
  for (j=0; j<rows; j++) {
    conn->queue[identifier].separated[j]=(char **)malloc(columns*sizeof(char *));
  }
  temp=conn->queue[identifier].response;
  real_row = malloc(sizeof(char *) * rows);
  real_row[0] = temp;
  row_num = 1;
  j = 0;
  while (temp[j] != '\0') {
    if (temp[j] == '\n') {
      temp[j] = '\0';
      if (temp[j + 1] != '\0')
        real_row[row_num] = temp + j + 1;
        row_num++;
    }
    j++;
  }

  for (j=0; j<rows; j++) {
    str=real_row[j];
    if (str == NULL) break;
    offset=strlen(str)+2;
    temp2=temp+offset;
    temp=temp2;
    line_temp1=str;
    for (i=0; i<columns; i++) {
      line_temp2=strchr(line_temp1, ',');
      if (line_temp2 == NULL) {
        conn->queue[identifier].separated[j][i]=MCVE_midstr(line_temp1, 0, strlen(line_temp1));
        break;
      }
      conn->queue[identifier].separated[j][i]=MCVE_midstr(line_temp1, 0, strlen(line_temp1)-strlen(line_temp2));
      line_temp1=line_temp2+1;
    }
  }
  free(real_row);
  conn->queue[identifier].columns=columns;
  conn->queue[identifier].rows=rows-1;
  return(1);
}

char *MCVE_GetCommaDelimited(MCVE_CONN *myconn, long identifier)
{
  _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
  return(conn->queue[identifier].response);
}


char *MCVE_GetCell(MCVE_CONN *myconn, long identifier, char *column, long row)
{
  _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
  int i;
  for (i=0; i<conn->queue[identifier].columns; i++) {
    if (strcasecmp(column, conn->queue[identifier].separated[0][i]) == 0)
      return(conn->queue[identifier].separated[row+1][i]);
  }
  return(NULL);
}

char *MCVE_GetCellByNum(MCVE_CONN *myconn, long identifier, int column, long row)
{
  _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
  return(conn->queue[identifier].separated[row+1][column]);
}

int MCVE_NumColumns(MCVE_CONN *myconn, long identifier)
{
  _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
  return(conn->queue[identifier].columns);
}

long MCVE_NumRows(MCVE_CONN *myconn, long identifier)
{
  _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
  return(conn->queue[identifier].rows);
}

char *MCVE_GetHeader(MCVE_CONN *myconn, long identifier, int column_num)
{
  _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
  return(conn->queue[identifier].separated[0][column_num]);
}




/* TEXT RESPONSE CONVERSIONS */

char *MCVE_TEXT_Code(int code)
{
  char *text=NULL;
  switch (code) {
    case MCVE_AUTH:
      text="AUTH";
    break;
    case MCVE_DENY:
      text="DENY";
    break;
    case MCVE_DUPL:
      text="DUPLICATE TRANS";
    break;
    case MCVE_CALL:
      text="CALL PROCESSOR";
    break;
    case MCVE_PKUP:
      text="PICKUP CARD";
    break;
    case MCVE_RETRY:
      text="RETRY";
    break;
    case MCVE_SETUP:
      text="SETUP";
    break;
    case MCVE_TIMEOUT:
      text="TIMEOUT";
    break;
    case MCVE_SUCCESS:
      text="SUCCESS";
    break;
    case MCVE_FAIL:
      text="FAIL";
    break;
    case MCVE_ERROR:
      text="ERROR";
    break;
    default:
      text="UNKNOWN";
    break;
  }
  return(text);
}

char *MCVE_TEXT_AVS(int code)
{
  char *text=NULL;
  switch(code) {
    case MCVE_STREET:
      text="STREET FAILED";
    break;
    case MCVE_ZIP:
      text="ZIP FAILED";
    break;
    case MCVE_GOOD:
      text="GOOD";
    break;
    case MCVE_BAD:
      text="BAD";
    break;
    case MCVE_UNKNOWN:
      text="UNKNOWN";
    break;
    default:
      text="UNKNOWN";
    break;
  } 
  return(text);
}

char *MCVE_TEXT_CV(int code)
{
 return(MCVE_TEXT_AVS(code));
}








