/* MCVE v3.x C API
   (c) 2002 by Main Street Softworks, Inc.
   Written by Brad House

   This API is being released to the public domain to be modified and used in any manor the holder of this code sees fit.
   For any questions please contact  support@mainstreetsoftworks.com
*/

#include "libmcve_main.h"
#include "mcve.h"

/* TRANSACTION ROUTINES */


long MCVE_Sale(MCVE_CONN *myconn, char *username, char *password, char *trackdata, char *account, char *expdate, double amount,
                               char *street, char *zip, char *cv, char *comments, char *clerkid, char *stationid, long ptrannum)
{
 // _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
  long identifier;

  if (username == NULL || password == NULL || (trackdata == NULL && (account == NULL || expdate == NULL)) || amount <= 0) {
    return(MCVE_ERROR);
  }
  identifier=MCVE_TransNew(myconn);
  MCVE_TransParam(myconn, identifier, MC_TRANTYPE, MC_TRAN_SALE);
  MCVE_TransParam(myconn, identifier, MC_USERNAME, username);
  MCVE_TransParam(myconn, identifier, MC_PASSWORD, password);
  MCVE_TransParam(myconn, identifier, MC_TRACKDATA, trackdata);
  MCVE_TransParam(myconn, identifier, MC_ACCOUNT, account);
  MCVE_TransParam(myconn, identifier, MC_EXPDATE, expdate);
  MCVE_TransParam(myconn, identifier, MC_STREET, street);
  MCVE_TransParam(myconn, identifier, MC_ZIP, zip);
  MCVE_TransParam(myconn, identifier, MC_CV, cv);
  MCVE_TransParam(myconn, identifier, MC_AMOUNT, amount);
  MCVE_TransParam(myconn, identifier, MC_COMMENTS, comments);
  MCVE_TransParam(myconn, identifier, MC_CLERKID, clerkid);
  MCVE_TransParam(myconn, identifier, MC_STATIONID, stationid);
  if (ptrannum != -1)
    MCVE_TransParam(myconn, identifier, MC_PTRANNUM, ptrannum);

  if (!MCVE_TransSend(myconn, identifier))
    return(-1);
  return(identifier);
}

long MCVE_PreAuth(MCVE_CONN *myconn, char *username, char *password, char *trackdata, char *account, char *expdate, double amount,
                                     char *street, char *zip, char *cv, char *comments, char *clerkid, char *stationid, long ptrannum)
{
 // _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
  long identifier;

  if (username == NULL || password == NULL || (trackdata == NULL && (account == NULL || expdate == NULL)) || amount <= 0)
    return(MCVE_ERROR);


  identifier=MCVE_TransNew(myconn);
  MCVE_TransParam(myconn, identifier, MC_TRANTYPE, MC_TRAN_PREAUTH);
  MCVE_TransParam(myconn, identifier, MC_USERNAME, username);
  MCVE_TransParam(myconn, identifier, MC_PASSWORD, password);
  MCVE_TransParam(myconn, identifier, MC_TRACKDATA, trackdata);
  MCVE_TransParam(myconn, identifier, MC_ACCOUNT, account);
  MCVE_TransParam(myconn, identifier, MC_EXPDATE, expdate);
  MCVE_TransParam(myconn, identifier, MC_STREET, street);
  MCVE_TransParam(myconn, identifier, MC_ZIP, zip);
  MCVE_TransParam(myconn, identifier, MC_CV, cv);
  MCVE_TransParam(myconn, identifier, MC_AMOUNT, amount);
  MCVE_TransParam(myconn, identifier, MC_COMMENTS, comments);
  MCVE_TransParam(myconn, identifier, MC_CLERKID, clerkid);
  MCVE_TransParam(myconn, identifier, MC_STATIONID, stationid);
  if (ptrannum != -1)
    MCVE_TransParam(myconn, identifier, MC_PTRANNUM, ptrannum);

  if (!MCVE_TransSend(myconn, identifier))
    return(-1);
  return(identifier);
}


long MCVE_Void(MCVE_CONN *myconn, char *username, char *password, MCVE_int64 ttid, long ptrannum)
{
 // _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
  long identifier;

  if (username == NULL || password == NULL || (ttid == -1 && ptrannum == -1))
    return(MCVE_ERROR);

  identifier=MCVE_TransNew(myconn);
  MCVE_TransParam(myconn, identifier, MC_TRANTYPE, MC_TRAN_VOID);
  MCVE_TransParam(myconn, identifier, MC_USERNAME, username);
  MCVE_TransParam(myconn, identifier, MC_PASSWORD, password);

  if (ttid != -1)
    MCVE_TransParam(myconn, identifier, MC_TTID, ttid);
  else if (ptrannum != -1)
    MCVE_TransParam(myconn, identifier, MC_PTRANNUM, ptrannum);

  if (!MCVE_TransSend(myconn, identifier))
    return(-1);
  return(identifier);
}


long MCVE_PreAuthCompletion(MCVE_CONN *myconn, char *username, char *password, double finalamount, MCVE_int64 ttid, long ptrannum)
{
 // _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
  long identifier;

  if (username == NULL || password == NULL || finalamount <= 0 || (ttid == -1 && ptrannum == -1))
    return(MCVE_ERROR);

  identifier=MCVE_TransNew(myconn);
  MCVE_TransParam(myconn, identifier, MC_TRANTYPE, MC_TRAN_PREAUTHCOMPLETE);
  MCVE_TransParam(myconn, identifier, MC_USERNAME, username);
  MCVE_TransParam(myconn, identifier, MC_PASSWORD, password);
  MCVE_TransParam(myconn, identifier, MC_AMOUNT, finalamount);
  if (ttid != -1)
    MCVE_TransParam(myconn, identifier, MC_TTID, ttid);
  else if (ptrannum != -1)
    MCVE_TransParam(myconn, identifier, MC_PTRANNUM, ptrannum);

  if (!MCVE_TransSend(myconn, identifier))
    return(-1);
  return(identifier);
}

long MCVE_Force(MCVE_CONN *myconn, char *username, char *password, char *trackdata, char *account, char *expdate, double amount,
                char *authcode, char *comments, char *clerkid, char *stationid, long ptrannum)
{
 // _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
  long identifier;

  if (username == NULL || password == NULL || (trackdata == NULL && (account == NULL || expdate == NULL)) || amount <= 0)
    return(MCVE_ERROR);

  identifier=MCVE_TransNew(myconn);
  MCVE_TransParam(myconn, identifier, MC_TRANTYPE, MC_TRAN_FORCE);
  MCVE_TransParam(myconn, identifier, MC_USERNAME, username);
  MCVE_TransParam(myconn, identifier, MC_PASSWORD, password);
  MCVE_TransParam(myconn, identifier, MC_TRACKDATA, trackdata);
  MCVE_TransParam(myconn, identifier, MC_ACCOUNT, account);
  MCVE_TransParam(myconn, identifier, MC_EXPDATE, expdate);
  MCVE_TransParam(myconn, identifier, MC_AMOUNT, amount);
  MCVE_TransParam(myconn, identifier, MC_APPRCODE, authcode);
  MCVE_TransParam(myconn, identifier, MC_COMMENTS, comments);
  MCVE_TransParam(myconn, identifier, MC_CLERKID, clerkid);
  MCVE_TransParam(myconn, identifier, MC_STATIONID, stationid);
  if (ptrannum != -1)
    MCVE_TransParam(myconn, identifier, MC_PTRANNUM, ptrannum);

  if (!MCVE_TransSend(myconn, identifier))
    return(-1);
  return(identifier);
}

long MCVE_Return(MCVE_CONN *myconn, char *username, char *password, char *trackdata, char *account, char *expdate, double amount,
                                     char *comments, char *clerkid, char *stationid, long ptrannum)
{
 // _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
  long identifier;
  if (username == NULL || password == NULL || (trackdata == NULL && (account == NULL || expdate == NULL)) || amount <= 0)
    return(MCVE_ERROR);

  identifier=MCVE_TransNew(myconn);
  MCVE_TransParam(myconn, identifier, MC_TRANTYPE, MC_TRAN_RETURN);
  MCVE_TransParam(myconn, identifier, MC_USERNAME, username);
  MCVE_TransParam(myconn, identifier, MC_PASSWORD, password);
  MCVE_TransParam(myconn, identifier, MC_TRACKDATA, trackdata);
  MCVE_TransParam(myconn, identifier, MC_ACCOUNT, account);
  MCVE_TransParam(myconn, identifier, MC_EXPDATE, expdate);
  MCVE_TransParam(myconn, identifier, MC_AMOUNT, amount);
  MCVE_TransParam(myconn, identifier, MC_COMMENTS, comments);
  MCVE_TransParam(myconn, identifier, MC_CLERKID, clerkid);
  MCVE_TransParam(myconn, identifier, MC_STATIONID, stationid);
  if (ptrannum != -1)
    MCVE_TransParam(myconn, identifier, MC_PTRANNUM, ptrannum);

  if (!MCVE_TransSend(myconn, identifier))
    return(-1);
  return(identifier);
}


long MCVE_Override(MCVE_CONN *myconn, char *username, char *password, char *trackdata, char *account, char *expdate, double amount,
                                     char *street, char *zip, char *cv, char *comments, char *clerkid, char *stationid, long ptrannum)
{
 // _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
  long identifier;
  if (username == NULL || password == NULL || (trackdata == NULL && (account == NULL || expdate == NULL)) || amount <= 0)
    return(MCVE_ERROR);

  identifier=MCVE_TransNew(myconn);
  MCVE_TransParam(myconn, identifier, MC_TRANTYPE, MC_TRAN_OVERRIDE);
  MCVE_TransParam(myconn, identifier, MC_USERNAME, username);
  MCVE_TransParam(myconn, identifier, MC_PASSWORD, password);
  MCVE_TransParam(myconn, identifier, MC_TRACKDATA, trackdata);
  MCVE_TransParam(myconn, identifier, MC_ACCOUNT, account);
  MCVE_TransParam(myconn, identifier, MC_EXPDATE, expdate);
  MCVE_TransParam(myconn, identifier, MC_STREET, street);
  MCVE_TransParam(myconn, identifier, MC_ZIP, zip);
  MCVE_TransParam(myconn, identifier, MC_CV, cv);
  MCVE_TransParam(myconn, identifier, MC_AMOUNT, amount);
  MCVE_TransParam(myconn, identifier, MC_COMMENTS, comments);
  MCVE_TransParam(myconn, identifier, MC_CLERKID, clerkid);
  MCVE_TransParam(myconn, identifier, MC_STATIONID, stationid);
  if (ptrannum != -1)
    MCVE_TransParam(myconn, identifier, MC_PTRANNUM, ptrannum);

  if (!MCVE_TransSend(myconn, identifier))
    return(-1);
  return(identifier);
}


long MCVE_Settle(MCVE_CONN *myconn, char *username, char *password, char *batch)
{
 // _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
  long identifier;
  if (username == NULL || password == NULL)
    return(MCVE_ERROR);

  identifier=MCVE_TransNew(myconn);
  MCVE_TransParam(myconn, identifier, MC_TRANTYPE, MC_TRAN_SETTLE);
  MCVE_TransParam(myconn, identifier, MC_USERNAME, username);
  MCVE_TransParam(myconn, identifier, MC_PASSWORD, password);
  if (batch == NULL || !strlen(batch)) {
    MCVE_TransParam(myconn, identifier, MC_BATCH, "all");
  } else {
    MCVE_TransParam(myconn, identifier, MC_BATCH, batch);
  }

  if (!MCVE_TransSend(myconn, identifier))
    return(-1);
  return(identifier);
}

/* USER-LEVEL ADMINISTRATIVE REQUESTS */

long MCVE_Gut(MCVE_CONN *myconn, char *username, char *password, int type, char *acct, char *clerkid, char *stationid, char *comments,
                              double ptrannum, char *bdate, char *edate)
{
  //_MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
  long identifier;

  if (username == NULL || password == NULL)
    return(MCVE_ERROR);

  identifier=MCVE_TransNew(myconn);
  MCVE_TransParam(myconn, identifier, MC_TRANTYPE, MC_TRAN_ADMIN);
  MCVE_TransParam(myconn, identifier, MC_ADMIN, MC_ADMIN_GUT);
  MCVE_TransParam(myconn, identifier, MC_USERNAME, username);
  MCVE_TransParam(myconn, identifier, MC_PASSWORD, password);
  MCVE_TransParam(myconn, identifier, MC_AUDITTYPE, type);
  MCVE_TransParam(myconn, identifier, MC_ACCT, acct);
  MCVE_TransParam(myconn, identifier, MC_CLERKID, clerkid);
  MCVE_TransParam(myconn, identifier, MC_STATIONID, stationid);
  MCVE_TransParam(myconn, identifier, MC_COMMENTS, comments);
  MCVE_TransParam(myconn, identifier, MC_BDATE, bdate);
  MCVE_TransParam(myconn, identifier, MC_EDATE, edate);
  if (ptrannum != -1)
    MCVE_TransParam(myconn, identifier, MC_PTRANNUM, ptrannum);

  if (!MCVE_TransSend(myconn, identifier))
    return(-1);
  return(identifier);
}


long MCVE_Gl(MCVE_CONN *myconn, char *username, char *password, int type, char *acct, char *batch, char *clerkid, char *stationid, char *comments,
                              double ptrannum, char *bdate, char *edate)
{
  //_MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
  long identifier;

  if (username == NULL || password == NULL)
    return(MCVE_ERROR);

  identifier=MCVE_TransNew(myconn);
  MCVE_TransParam(myconn, identifier, MC_TRANTYPE, MC_TRAN_ADMIN);
  MCVE_TransParam(myconn, identifier, MC_ADMIN, MC_ADMIN_GL);
  MCVE_TransParam(myconn, identifier, MC_USERNAME, username);
  MCVE_TransParam(myconn, identifier, MC_PASSWORD, password);
  MCVE_TransParam(myconn, identifier, MC_AUDITTYPE, type);
  MCVE_TransParam(myconn, identifier, MC_ACCT, acct);
  MCVE_TransParam(myconn, identifier, MC_BATCH, batch);
  MCVE_TransParam(myconn, identifier, MC_CLERKID, clerkid);
  MCVE_TransParam(myconn, identifier, MC_STATIONID, stationid);
  MCVE_TransParam(myconn, identifier, MC_COMMENTS, comments);
  MCVE_TransParam(myconn, identifier, MC_BDATE, bdate);
  MCVE_TransParam(myconn, identifier, MC_EDATE, edate);
  if (ptrannum != -1)
    MCVE_TransParam(myconn, identifier, MC_PTRANNUM, ptrannum);

  if (!MCVE_TransSend(myconn, identifier))
    return(-1);
  return(identifier);
}

long MCVE_Gft(MCVE_CONN *myconn, char *username, char *password, int type, char *acct, char *clerkid, char *stationid, char *comments,
                              double ptrannum, char *bdate, char *edate)
{
  // _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
  long identifier;

  if (username == NULL || password == NULL)
    return(MCVE_ERROR);
    
  identifier=MCVE_TransNew(myconn);
  MCVE_TransParam(myconn, identifier, MC_TRANTYPE, MC_TRAN_ADMIN);
  MCVE_TransParam(myconn, identifier, MC_ADMIN, MC_ADMIN_GFT);
  MCVE_TransParam(myconn, identifier, MC_USERNAME, username);
  MCVE_TransParam(myconn, identifier, MC_PASSWORD, password);
  MCVE_TransParam(myconn, identifier, MC_AUDITTYPE, type);
  MCVE_TransParam(myconn, identifier, MC_ACCT, acct);
  MCVE_TransParam(myconn, identifier, MC_CLERKID, clerkid);
  MCVE_TransParam(myconn, identifier, MC_STATIONID, stationid);
  MCVE_TransParam(myconn, identifier, MC_COMMENTS, comments);
  MCVE_TransParam(myconn, identifier, MC_BDATE, bdate);
  MCVE_TransParam(myconn, identifier, MC_EDATE, edate);
  if (ptrannum != -1)
    MCVE_TransParam(myconn, identifier, MC_PTRANNUM, ptrannum);

  if (!MCVE_TransSend(myconn, identifier))
    return(-1);
  return(identifier);
}

/*
long MCVE_Ub(MCVE_CONN *myconn, char *username, char *password)
{
  _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
  MCVE_TRANS trans;
  MCVE_ClearTransaction(&trans);
  if (username == NULL || password == NULL)
    return(MCVE_ERROR);
  trans.trans_type=MCVE_UB;
  trans.username=username;
  trans.password=password;
  return(MCVE_SendTransaction(myconn, trans));
}
*/

long MCVE_Rs(MCVE_CONN *myconn, char *username, char *password, char *stationid)
{
  //_MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
  long identifier;

  if (username == NULL || password == NULL || stationid == NULL)
    return(MCVE_ERROR);

  identifier=MCVE_TransNew(myconn);
  MCVE_TransParam(myconn, identifier, MC_TRANTYPE, MC_TRAN_ADMIN);
  MCVE_TransParam(myconn, identifier, MC_ADMIN, MC_ADMIN_RS);
  MCVE_TransParam(myconn, identifier, MC_USERNAME, username);
  MCVE_TransParam(myconn, identifier, MC_PASSWORD, password);
  MCVE_TransParam(myconn, identifier, MC_STATIONID, stationid);

  if (!MCVE_TransSend(myconn, identifier))
    return(-1);
  return(identifier);
}

long MCVE_Qc(MCVE_CONN *myconn, char *username, char *password, char *clerkid, char *stationid, char *comments, double ptrannum)
{
 // _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
  long identifier;

  if (username == NULL || password == NULL)
    return(MCVE_ERROR);

  identifier=MCVE_TransNew(myconn);
  MCVE_TransParam(myconn, identifier, MC_TRANTYPE, MC_TRAN_ADMIN);
  MCVE_TransParam(myconn, identifier, MC_ADMIN, MC_ADMIN_QC);
  MCVE_TransParam(myconn, identifier, MC_USERNAME, username);
  MCVE_TransParam(myconn, identifier, MC_PASSWORD, password);
  MCVE_TransParam(myconn, identifier, MC_CLERKID, clerkid);
  MCVE_TransParam(myconn, identifier, MC_STATIONID, stationid);
  MCVE_TransParam(myconn, identifier, MC_COMMENTS, comments);
  if (ptrannum != -1)
    MCVE_TransParam(myconn, identifier, MC_PTRANNUM, ptrannum);

  if (!MCVE_TransSend(myconn, identifier))
    return(-1);
  return(identifier);

}

long MCVE_Chkpwd(MCVE_CONN *myconn, char *username, char *password)
{
  //_MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
  long identifier;

  if (username == NULL || password == NULL)
    return(MCVE_ERROR);

  identifier=MCVE_TransNew(myconn);
  MCVE_TransParam(myconn, identifier, MC_TRANTYPE, MC_TRAN_CHKPWD);
  MCVE_TransParam(myconn, identifier, MC_USERNAME, username);
  MCVE_TransParam(myconn, identifier, MC_PASSWORD, password);

  if (!MCVE_TransSend(myconn, identifier))
    return(-1);
  return(identifier);
}

long MCVE_Bt(MCVE_CONN *myconn, char *username, char *password)
{
  //_MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
  long identifier;
  if (username == NULL || password == NULL)
    return(MCVE_ERROR);

  identifier=MCVE_TransNew(myconn);
  MCVE_TransParam(myconn, identifier, MC_TRANTYPE, MC_TRAN_ADMIN);
  MCVE_TransParam(myconn, identifier, MC_ADMIN, MC_ADMIN_BT);
  MCVE_TransParam(myconn, identifier, MC_USERNAME, username);
  MCVE_TransParam(myconn, identifier, MC_PASSWORD, password);

  if (!MCVE_TransSend(myconn, identifier))
    return(-1);
  return(identifier);
}

/* ENGINE-ADMIN REQUESTS */


long MCVE_Chngpwd(MCVE_CONN *myconn, char *adminpassword, char *newpassword)
{
 // _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
  long identifier;
  if (adminpassword == NULL || newpassword == NULL)
    return(MCVE_ERROR);

  identifier=MCVE_TransNew(myconn);
  MCVE_TransParam(myconn, identifier, MC_TRANTYPE, MC_TRAN_CHNGPWD);
  MCVE_TransParam(myconn, identifier, MC_USERNAME, "MCVEADMIN");
  MCVE_TransParam(myconn, identifier, MC_PASSWORD, adminpassword);
  MCVE_TransParam(myconn, identifier, MC_PWD, newpassword);
  if (!MCVE_TransSend(myconn, identifier))
    return(-1);
  return(identifier);
}

long MCVE_ListUsers(MCVE_CONN *myconn, char *password)
{
  //  _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
  long identifier;
  if (password == NULL)
    return(MCVE_ERROR);
  identifier=MCVE_TransNew(myconn);
  MCVE_TransParam(myconn, identifier, MC_TRANTYPE, MC_TRAN_LISTUSERS);
  MCVE_TransParam(myconn, identifier, MC_USERNAME, "MCVEADMIN");
  MCVE_TransParam(myconn, identifier, MC_PASSWORD, password);
  if (!MCVE_TransSend(myconn, identifier))
    return(-1);
  return(identifier);
}

long MCVE_EnableUser(MCVE_CONN *myconn, char *adminpwd, char *user)
{
  // _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
  long identifier;
  if (adminpwd == NULL || user == NULL)
    return(MCVE_ERROR);
  identifier=MCVE_TransNew(myconn);
  MCVE_TransParam(myconn, identifier, MC_TRANTYPE, MC_TRAN_ENABLEUSER);
  MCVE_TransParam(myconn, identifier, MC_USERNAME, "MCVEADMIN");
  MCVE_TransParam(myconn, identifier, MC_PASSWORD, adminpwd);
  MCVE_TransParam(myconn, identifier, MC_USER, user);
  if (!MCVE_TransSend(myconn, identifier))
    return(-1);
  return(identifier);
}

long MCVE_DisableUser(MCVE_CONN *myconn, char *adminpwd, char *user)
{
  // _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
  long identifier;
  if (adminpwd == NULL || user == NULL)
    return(MCVE_ERROR);
  identifier=MCVE_TransNew(myconn);
  MCVE_TransParam(myconn, identifier, MC_TRANTYPE, MC_TRAN_DISABLEUSER);
  MCVE_TransParam(myconn, identifier, MC_USERNAME, "MCVEADMIN");
  MCVE_TransParam(myconn, identifier, MC_PASSWORD, adminpwd);
  MCVE_TransParam(myconn, identifier, MC_USER, user);
  if (!MCVE_TransSend(myconn, identifier))
    return(-1);
  return(identifier);
}


long MCVE_GetUserInfo(MCVE_CONN *myconn, char *adminpwd, char *user)
{
  long identifier;
  if (adminpwd == NULL || user == NULL)
    return(MCVE_ERROR);
  identifier=MCVE_TransNew(myconn);
  MCVE_TransParam(myconn, identifier, MC_TRANTYPE, MC_TRAN_GETUSERINFO);
  MCVE_TransParam(myconn, identifier, MC_USERNAME, "MCVEADMIN");
  MCVE_TransParam(myconn, identifier, MC_PASSWORD, adminpwd);
  MCVE_TransParam(myconn, identifier, MC_USER, user);
  if (!MCVE_TransSend(myconn, identifier))
    return(-1);
  return(identifier);
}

long MCVE_DelUser(MCVE_CONN *myconn, char *password, char *user)
{
  long identifier;
  if (password == NULL || user == NULL)
    return(MCVE_ERROR);
  identifier=MCVE_TransNew(myconn);
  MCVE_TransParam(myconn, identifier, MC_TRANTYPE, MC_TRAN_DELUSER);
  MCVE_TransParam(myconn, identifier, MC_USERNAME, "MCVEADMIN");
  MCVE_TransParam(myconn, identifier, MC_PASSWORD, password);
  MCVE_TransParam(myconn, identifier, MC_USER, user);
  if (!MCVE_TransSend(myconn, identifier))
    return(-1);
  return(identifier);
}

long MCVE_ListStats(MCVE_CONN *myconn, char *adminpwd)
{
  //_MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
  long identifier;
  if (adminpwd == NULL)
    return(MCVE_ERROR);
  identifier=MCVE_TransNew(myconn);
  MCVE_TransParam(myconn, identifier, MC_TRANTYPE, MC_TRAN_LISTSTATS);
  MCVE_TransParam(myconn, identifier, MC_USERNAME, "MCVEADMIN");
  MCVE_TransParam(myconn, identifier, MC_PASSWORD, adminpwd);
  if (!MCVE_TransSend(myconn, identifier))
    return(-1);
  return(identifier);
}

long MCVE_Export(MCVE_CONN *myconn, char *adminpwd, char *filename)
{
 // _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
  long identifier;
  if (filename == NULL || adminpwd == NULL)
    return(MCVE_ERROR);
  identifier=MCVE_TransNew(myconn);
  MCVE_TransParam(myconn, identifier, MC_TRANTYPE, MC_TRAN_EXPORT);
  MCVE_TransParam(myconn, identifier, MC_USERNAME, "MCVEADMIN");
  MCVE_TransParam(myconn, identifier, MC_PASSWORD, adminpwd);
  MCVE_TransParam(myconn, identifier, MC_FILE, filename);
  if (!MCVE_TransSend(myconn, identifier))
    return(-1);
  return(identifier);
}

long MCVE_Import(MCVE_CONN *myconn, char *adminpwd, char *filename)
{
 // _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
  long identifier;
  if (filename == NULL || adminpwd == NULL)
    return(MCVE_ERROR);
  identifier=MCVE_TransNew(myconn);
  MCVE_TransParam(myconn, identifier, MC_TRANTYPE, MC_TRAN_IMPORT);
  MCVE_TransParam(myconn, identifier, MC_USERNAME, "MCVEADMIN");
  MCVE_TransParam(myconn, identifier, MC_PASSWORD, adminpwd);
  MCVE_TransParam(myconn, identifier, MC_FILE, filename);
  if (!MCVE_TransSend(myconn, identifier))
    return(-1);
  return(identifier);
}

/* LEGACY USER FUNCTIONS */

typedef struct _UserSetup {
  char *proc;
  char *user;
  char *pwd;
  char *indcode;
  char *merchid;
  char *bankid;
  char *termid;
  char *clientnum;
  char *storeid;
  char *agentid;
  char *chainid;
  char *zipcode;
  char *timezone;
  char *merchcat;
  char *mername;
  char *merchloc;
  char *statecode;
  char *servicephone;
} _UserSetup;

void MCVE_InitUserSetup(MCVE_UserSetup *setup)
{

  _UserSetup *usersetup;
  setup[0]=(MCVE_UserSetup)malloc(sizeof(_UserSetup)*1);
  usersetup=(_UserSetup *)setup[0];
  usersetup->proc=NULL;
  usersetup->user=NULL;
  usersetup->pwd=NULL;
  usersetup->indcode=NULL;
  usersetup->merchid=NULL;
  usersetup->bankid=NULL;
  usersetup->termid =NULL;
  usersetup->clientnum =NULL;
  usersetup->storeid =NULL;
  usersetup->agentid =NULL;
  usersetup->chainid =NULL;
  usersetup->zipcode =NULL;
  usersetup->timezone =NULL;
  usersetup->merchcat =NULL;
  usersetup->mername =NULL;
  usersetup->merchloc =NULL;
  usersetup->statecode =NULL;
  usersetup->servicephone =NULL;
}

int MCVE_AddUserArg(MCVE_UserSetup *setup, int argtype, char *argval)
{
  _UserSetup *usersetup=(_UserSetup *)setup[0];

  char **field;

  if (argtype == MCVE_PROC)
    field=&usersetup->proc;
  else if (argtype == MCVE_USER)
    field=&usersetup->user;
  else if (argtype == MCVE_PWD)
    field=&usersetup->pwd;
  else if (argtype == MCVE_INDCODE)
    field=&usersetup->indcode;
  else if (argtype == MCVE_MERCHID)
    field=&usersetup->merchid;
  else if (argtype == MCVE_BANKID)
    field=&usersetup->bankid;
  else if (argtype == MCVE_TERMID)
    field=&usersetup->termid;
  else if (argtype == MCVE_CLIENTNUM)
    field=&usersetup->clientnum;
  else if (argtype == MCVE_STOREID)
    field=&usersetup->storeid;
  else if (argtype == MCVE_AGENTID)
    field=&usersetup->agentid;
  else if (argtype == MCVE_CHAINID)
    field=&usersetup->chainid;
  else if (argtype == MCVE_ZIPCODE)
    field=&usersetup->zipcode;
  else if (argtype == MCVE_TIMEZONE)
    field=&usersetup->timezone;
  else if (argtype == MCVE_MERCHCAT)
    field=&usersetup->merchcat;
  else if (argtype == MCVE_MERNAME)
    field=&usersetup->mername;
  else if (argtype == MCVE_MERCHLOC)
    field=&usersetup->merchloc;
  else if (argtype == MCVE_STATECODE)
    field=&usersetup->statecode;
  else if (argtype == MCVE_SERVICEPHONE)
    field=&usersetup->servicephone;
  else
    return(0);

  field[0]=(char *)malloc((strlen(argval)+1)*sizeof(char));
  strcpy(field[0], argval);
  return(1);
}


void MCVE_DeleteUserSetup(MCVE_UserSetup *setup)
{
  _UserSetup *usersetup=(_UserSetup *)setup[0];

  if (usersetup->proc != NULL) {
    free(usersetup->proc);
    usersetup->proc=NULL;
  }
  if (usersetup->user != NULL) {
    free(usersetup->user);
    usersetup->user=NULL;
  }
  if (usersetup->pwd != NULL) {
    free(usersetup->pwd);
    usersetup->pwd=NULL;
  }
  if (usersetup->indcode != NULL) {
    free(usersetup->indcode);
    usersetup->indcode=NULL;
  }
  if (usersetup->merchid != NULL) {
    free(usersetup->merchid);
    usersetup->merchid=NULL;
  }
  if (usersetup->bankid != NULL) {
    free(usersetup->bankid);
    usersetup->bankid=NULL;
  }
  if (usersetup->termid != NULL) {
    free(usersetup->termid);
    usersetup->termid=NULL;
  }
  if (usersetup->clientnum != NULL) {
    free(usersetup->clientnum);
    usersetup->clientnum=NULL;
  }
  if (usersetup->storeid != NULL) {
    free(usersetup->storeid);
    usersetup->storeid=NULL;
  }
  if (usersetup->agentid != NULL) {
    free(usersetup->agentid);
    usersetup->agentid=NULL;
  }
  if (usersetup->chainid != NULL) {
    free(usersetup->chainid);
    usersetup->chainid=NULL;
  }
  if (usersetup->zipcode != NULL) {
    free(usersetup->zipcode);
    usersetup->zipcode=NULL;
  }
  if (usersetup->timezone != NULL) {
    free(usersetup->timezone);
    usersetup->timezone=NULL;
  }
  if (usersetup->merchcat != NULL) {
    free(usersetup->merchcat);
    usersetup->merchcat=NULL;
  }
  if (usersetup->mername != NULL) {
    free(usersetup->mername);
    usersetup->mername=NULL;
  }
  if (usersetup->merchloc != NULL) {
    free(usersetup->merchloc);
    usersetup->merchloc=NULL;
  }
  if (usersetup->statecode != NULL) {
    free(usersetup->statecode);
    usersetup->statecode=NULL;
  }
  if (usersetup->servicephone != NULL) {
    free(usersetup->servicephone);
    usersetup->servicephone=NULL;
  }
  free(setup[0]);
}

long MCVE_AddUser(MCVE_CONN *myconn, char *adminpwd, MCVE_UserSetup *setup)
{
  _UserSetup *usersetup=(_UserSetup *)setup[0];
  //_MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
  long identifier;

  identifier=MCVE_TransNew(myconn);
  MCVE_TransParam(myconn, identifier, MC_TRANTYPE, MC_TRAN_ADDUSER);
  MCVE_TransParam(myconn, identifier, MC_USERNAME, "MCVEADMIN");
  MCVE_TransParam(myconn, identifier, MC_PASSWORD, adminpwd);

  if (usersetup->proc != NULL) {
    MCVE_TransParam(myconn, identifier, MC_USER_PROC, usersetup->proc);
  }
  if (usersetup->user != NULL) {
    MCVE_TransParam(myconn, identifier, MC_USER_USER, usersetup->user);
  }
  if (usersetup->pwd != NULL) {
    MCVE_TransParam(myconn, identifier, MC_USER_PWD, usersetup->pwd);
  }
  if (usersetup->indcode != NULL) {
    MCVE_TransParam(myconn, identifier, MC_USER_INDCODE, usersetup->indcode);
  }
  if (usersetup->merchid != NULL) {
    MCVE_TransParam(myconn, identifier, MC_USER_MERCHID, usersetup->merchid);
  }
  if (usersetup->bankid != NULL) {
    MCVE_TransParam(myconn, identifier, MC_USER_BANKID, usersetup->bankid);
  }
  if (usersetup->termid != NULL) {
    MCVE_TransParam(myconn, identifier, MC_USER_TERMID, usersetup->termid);
  }
  if (usersetup->clientnum != NULL) {
    MCVE_TransParam(myconn, identifier, MC_USER_CLIENTNUM, usersetup->clientnum);
  }
  if (usersetup->storeid != NULL) {
    MCVE_TransParam(myconn, identifier, MC_USER_STOREID, usersetup->storeid);
  }
  if (usersetup->agentid != NULL) {
    MCVE_TransParam(myconn, identifier, MC_USER_AGENTID, usersetup->agentid);
  }
  if (usersetup->chainid != NULL) {
    MCVE_TransParam(myconn, identifier, MC_USER_CHAINID, usersetup->chainid);
  }
  if (usersetup->zipcode != NULL) {
    MCVE_TransParam(myconn, identifier, MC_USER_ZIPCODE, usersetup->zipcode);
  }
  if (usersetup->timezone != NULL) {
    MCVE_TransParam(myconn, identifier, MC_USER_TIMEZONE, usersetup->timezone);
  }
  if (usersetup->merchcat != NULL) {
    MCVE_TransParam(myconn, identifier, MC_USER_MERCHCAT, usersetup->merchcat);
  }
  if (usersetup->mername != NULL) {
    MCVE_TransParam(myconn, identifier, MC_USER_MERNAME, usersetup->mername);
  }
  if (usersetup->merchloc != NULL) {
    MCVE_TransParam(myconn, identifier, MC_USER_MERCHLOC, usersetup->merchloc);
  }
  if (usersetup->statecode != NULL) {
    MCVE_TransParam(myconn, identifier, MC_USER_STATECODE, usersetup->statecode);
  }
  if (usersetup->servicephone != NULL) {
    MCVE_TransParam(myconn, identifier, MC_USER_PHONE, usersetup->servicephone);
  }

  if (!MCVE_TransSend(myconn, identifier))
    return(-1);
  return(identifier);
}

char *MCVE_GetUserArg(MCVE_UserSetup *setup, int argtype)
{
  _UserSetup *usersetup=(_UserSetup *)setup[0];
  char *field;

  if (argtype == MCVE_PROC)
    field=usersetup->proc;
  else if (argtype == MCVE_USER)
    field=usersetup->user;
  else if (argtype == MCVE_PWD)
    field=usersetup->pwd;
  else if (argtype == MCVE_INDCODE)
    field=usersetup->indcode;
  else if (argtype == MCVE_MERCHID)
    field=usersetup->merchid;
  else if (argtype == MCVE_BANKID)
    field=usersetup->bankid;
  else if (argtype == MCVE_TERMID)
    field=usersetup->termid;
  else if (argtype == MCVE_CLIENTNUM)
    field=usersetup->clientnum;
  else if (argtype == MCVE_STOREID)
    field=usersetup->storeid;
  else if (argtype == MCVE_AGENTID)
    field=usersetup->agentid;
  else if (argtype == MCVE_CHAINID)
    field=usersetup->chainid;
  else if (argtype == MCVE_ZIPCODE)
    field=usersetup->zipcode;
  else if (argtype == MCVE_TIMEZONE)
    field=usersetup->timezone;
  else if (argtype == MCVE_MERCHCAT)
    field=usersetup->merchcat;
  else if (argtype == MCVE_MERNAME)
    field=usersetup->mername;
  else if (argtype == MCVE_MERCHLOC)
    field=usersetup->merchloc;
  else if (argtype == MCVE_STATECODE)
    field=usersetup->statecode;
  else if (argtype == MCVE_SERVICEPHONE)
    field=usersetup->servicephone;
  else
    return(NULL);
  return(field);
}

long MCVE_EditUser(MCVE_CONN *myconn, char *adminpwd, MCVE_UserSetup *setup)
{
  _UserSetup *usersetup=(_UserSetup *)setup[0];
  //_MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
  long identifier;

  identifier=MCVE_TransNew(myconn);
  MCVE_TransParam(myconn, identifier, MC_TRANTYPE, MC_TRAN_EDITUSER);
  MCVE_TransParam(myconn, identifier, MC_USERNAME, "MCVEADMIN");
  MCVE_TransParam(myconn, identifier, MC_PASSWORD, adminpwd);

  if (usersetup->proc != NULL) {
    MCVE_TransParam(myconn, identifier, MC_USER_PROC, usersetup->proc);
  }
  if (usersetup->user != NULL) {
    MCVE_TransParam(myconn, identifier, MC_USER_USER, usersetup->user);
  }
  if (usersetup->pwd != NULL) {
    MCVE_TransParam(myconn, identifier, MC_USER_PWD, usersetup->pwd);
  }
  if (usersetup->indcode != NULL) {
    MCVE_TransParam(myconn, identifier, MC_USER_INDCODE, usersetup->indcode);
  }
  if (usersetup->merchid != NULL) {
    MCVE_TransParam(myconn, identifier, MC_USER_MERCHID, usersetup->merchid);
  }
  if (usersetup->bankid != NULL) {
    MCVE_TransParam(myconn, identifier, MC_USER_BANKID, usersetup->bankid);
  }
  if (usersetup->termid != NULL) {
    MCVE_TransParam(myconn, identifier, MC_USER_TERMID, usersetup->termid);
  }
  if (usersetup->clientnum != NULL) {
    MCVE_TransParam(myconn, identifier, MC_USER_CLIENTNUM, usersetup->clientnum);
  }
  if (usersetup->storeid != NULL) {
    MCVE_TransParam(myconn, identifier, MC_USER_STOREID, usersetup->storeid);
  }
  if (usersetup->agentid != NULL) {
    MCVE_TransParam(myconn, identifier, MC_USER_AGENTID, usersetup->agentid);
  }
  if (usersetup->chainid != NULL) {
    MCVE_TransParam(myconn, identifier, MC_USER_CHAINID, usersetup->chainid);
  }
  if (usersetup->zipcode != NULL) {
    MCVE_TransParam(myconn, identifier, MC_USER_ZIPCODE, usersetup->zipcode);
  }
  if (usersetup->timezone != NULL) {
    MCVE_TransParam(myconn, identifier, MC_USER_TIMEZONE, usersetup->timezone);
  }
  if (usersetup->merchcat != NULL) {
    MCVE_TransParam(myconn, identifier, MC_USER_MERCHCAT, usersetup->merchcat);
  }
  if (usersetup->mername != NULL) {
    MCVE_TransParam(myconn, identifier, MC_USER_MERNAME, usersetup->mername);
  }
  if (usersetup->merchloc != NULL) {
    MCVE_TransParam(myconn, identifier, MC_USER_MERCHLOC, usersetup->merchloc);
  }
  if (usersetup->statecode != NULL) {
    MCVE_TransParam(myconn, identifier, MC_USER_STATECODE, usersetup->statecode);
  }
  if (usersetup->servicephone != NULL) {
    MCVE_TransParam(myconn, identifier, MC_USER_PHONE, usersetup->servicephone);
  }

  if (!MCVE_TransSend(myconn, identifier))
    return(-1);
  return(identifier);
}

