/* MCVE v3.x C API
   (c) 2002 by Main Street Softworks, Inc.
   Written by Brad House

   This API is being released to the public domain to be modified and used in any manor the holder of this code sees fit.
   For any questions please contact  support@mainstreetsoftworks.com
*/

#ifndef __LIBMCVE_MAIN_H__
#define __LIBMCVE_MAIN_H__

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

/* Standard includes ... all platforms */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <sys/types.h>
#include <stdarg.h>
#include <errno.h>
#include <fcntl.h>

#ifdef HAVE_STRINGS_H
#include <strings.h>
#endif

/* Win32 includes */
#ifdef WIN32
#include <io.h>
#include <winsock2.h>
#include <sys/timeb.h>
#else /* !Win32 */
/* UNIX/POSIX includes */
#include <unistd.h>
#include <sys/socket.h>
#ifndef DISABLE_IP
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#endif
#include <sys/time.h>
#include <dirent.h>
#ifdef MCVE_PTHREAD
#include <pthread.h>
#endif /* MCVE_PTHREAD */

#endif /* Win32 */


#ifdef ENABLE_SSL
#include <openssl/crypto.h>
#include <openssl/x509.h>
#include <openssl/pem.h>
#include <openssl/ssl.h>
#include <openssl/err.h>
#include <openssl/rand.h>
#endif

#ifndef HAVE_SOCKLEN_T
typedef int socklen_t;
#endif

#include "libmcve_utils.h"
#include "libmcve_misc.h"
#include "libmcve_parsing.h"
#include "libmcve_cdparsing.h"
#include "libmcve_locking.h"
#include "libmcve_sockets.h"
#include "libmcve_file.h"
#include "libmcve_strings.h"
#include "libmcve_snprintf.h"


#ifdef WIN32
#define ioctl(a,b,c) ioctlsocket(a,b,c)
#endif


#define MCVE_FILE 1
#define MCVE_SOCKETS 2
#define MCVE_SSL 3



#define MCVE_LOCK_IN 1
#define MCVE_LOCK_OUT 2
#define MCVE_LOCK_QUEUE 3

#include "mcve.h"

typedef struct MCVE_TRANS {
  char *key;
  char *value;
} MCVE_TRANS;

typedef struct MCVE_QUEUE {
  char identifier[31];
  int type;
  int admin;
  int status;

  /* Store outgoing transaction fields -- key/value pairs */
  int transaction_fields;
  MCVE_TRANS *transaction;


  // Stuff read from return
  int code;
  int avs;
  int cv;
  char *auth;
  char *text;
  MCVE_int64 tid;
  char *item;
  char *batch;
  
  int resp_fields;
  MCVE_TRANS *parsed_resp;
  // Comma Delimited response storage
  char *response;
  int iscommadelimiated;
  // storage
  char ***separated;
  int columns;
  int rows;
} MCVE_QUEUE;

#define MCVE_DISCONNECTED 0
#define MCVE_CONNECTED 1

typedef struct _MCVE_CONN {
  int conn_method;  // Drop File, Sockets, SSL
  char location[251]; // Either Drop-File directory, or hostname
  unsigned long port;  // For IP and SSL, a port number
  int ptr;  // For IP and SSL, the pointer to an open connection
  char *inbuf;
  char *outbuf;
  
  int status;

  long timeout;
  int blocking;
  int verifyconn;
  int verifyssl;
  int max_conn_time;
  char *error_text;
#ifdef MCVE_THREADSAFE
#ifdef MCVE_PTHREAD
  pthread_mutex_t in_lock;
  pthread_mutex_t out_lock;
  pthread_mutex_t queue_lock;
#else
  int in_lock;
  int out_lock;
  int queue_lock;
#endif // MCVE_PTHREAD
#endif // MCVE_THREADSAFE

#ifdef ENABLE_SSL
  SSL *ssl;
  X509 *server_cert;
#endif
  // int CA_verify;
  // char *CA_cert;  // To store an SSL certificate
  // char *CA_key;  // To store an SSL key
  long outstanding_auths;
  long queue_length;
  MCVE_QUEUE *queue;
} _MCVE_CONN;


#ifdef NO_ATOLL
#define atoll(a) atol(a)
#endif


#ifdef ENABLE_SSL
extern SSL_CTX *client_ctx;
#endif

int MCVE_TransParam_Add(MCVE_CONN *myconn, long identifier, char *key, char *value);

#endif
