/* Modified by Main Street Softworks (Brad House & Chris Faulhaber)
   2003
   - Redesigned it to be thread-safe
   - vasprintf,asprintf,vfprintf added
   - Made SnprfOverflow actually work
   - Added support for %i
   - Prepended MCVE_ to function names
*/

/*
 * Copyright (c) 1983, 1995, 1996 Eric P. Allman
 * Copyright (c) 1988, 1993
 *	The Regents of the University of California.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *	  notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *	  notice, this list of conditions and the following disclaimer in the
 *	  documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *	  must display the following acknowledgement:
 *	This product includes software developed by the University of
 *	California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *	  may be used to endorse or promote products derived from this software
 *	  without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.	IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */


/*
**	SNPRINTF, VSNPRINT -- counted versions of printf
**
**	These versions have been grabbed off the net.  They have been
**	cleaned up to compile properly and support for .precision and
**	%lx has been added.
*/

/**************************************************************
 * Original:
 * Patrick Powell Tue Apr 11 09:48:21 PDT 1995
 * A bombproof version of doprnt (dopr) included.
 * Sigh.  This sort of thing is always nasty do deal with.	Note that
 * the version here does not include floating point. (now it does ... tgl)
 *
 * snprintf() is used instead of sprintf() as it does limit checks
 * for string length.  This covers a nasty loophole.
 *
 * The other functions are there to prevent NULL pointers from
 * causing nast effects.
 **************************************************************/


#include "mcve.h"
#include "libmcve_main.h"

#define MCVE_malloc(a) malloc(a)
#define MCVE_realloc(a,b) realloc(a,b)
#define MCVE_free(a) free(a)


static int MCVE_vsnprintf_internal(char *str, size_t count, const char *fmt, va_list args, int *overflow);

static void dopr(char *buffer, const char *format, va_list args, char *end, int *SnprfOverflow);


int
MCVE_snprintf(char *str, size_t count, const char *fmt,...)
{
	int			len;
	va_list		args;

	va_start(args, fmt);
	len = MCVE_vsnprintf(str, count, fmt, args);
	va_end(args);
	return len;
}


static int
MCVE_vsnprintf_internal(char *str, size_t count, const char *fmt, va_list args, int *overflow)
{
        char *end;
	int SnprfOverflow;
	str[0] = '\0';
	end = str + count - 1;
	SnprfOverflow = 0;
	dopr(str, fmt, args, end, &SnprfOverflow);

	if (overflow != NULL) {
	  *overflow=SnprfOverflow;
	}
	if (SnprfOverflow) {
	  end[0]='\0';
	}
	return strlen(str);
}

int MCVE_vsnprintf(char *str, size_t count, const char *fmt, va_list args)
{
  return(MCVE_vsnprintf_internal(str, count, fmt, args, NULL));
}

#define CHUNKSIZE 512

int MCVE_vasprintf(char **ret, const char *fmt, va_list args)
{
  int chunks;
  size_t buflen;
  char *buf;
  int len;
  int overflow=0;


  chunks = ((strlen(fmt) + 1) / CHUNKSIZE) + 1;
  buflen = chunks * CHUNKSIZE;

  /* SHOULD ONLY LOOP ONCE AS WE'RE GRABBING THE OVERFLOW FROM THE INTERNAL
     VSNPRINTF */
  while (1) {
    if ((buf = MCVE_malloc(buflen)) == NULL) {
      *ret = NULL;
      return -1;
    }
    overflow=0;
    len = MCVE_vsnprintf_internal(buf, buflen, fmt, args, &overflow);
    if (overflow) {
      MCVE_free(buf);
      buflen+=(overflow+1);
    } else {
      break;
    }
  }
  *ret = buf;
  return len;
}

int MCVE_asprintf(char **ret, const char *fmt, ...)
{
  int len;
  va_list ap;

  va_start(ap, fmt);
  len=MCVE_vasprintf(ret, fmt, ap);
  va_end(ap);
  return(len);
}

int MCVE_vfprintf(FILE *fp, const char *fmt, va_list args)
{
  char *ret=NULL;
  int retn;
  int len;

  len=MCVE_vasprintf(&ret, fmt, args);
  if (len == -1) return(-1);
  retn=fputs(ret, fp);
  MCVE_free(ret);
  if (retn < 0) return(-1);
  return(len);
}


/*
 * dopr(): poor man's version of doprintf
 */

static void fmtstr(char *value, int ljust, int len, int zpad, int maxwidth, int *SnprfOverflow, char **output, char *end);
static void fmtnum(long_long value, int base, int dosign, int ljust, int len, int zpad, int *SnprfOverflow, char **output, char *end);
static void fmtfloat(double value, char type, int ljust, int len, int precision, int pointflag, int *SnprfOverflow, char **output, char *end);
static void dostr(char *str, int cut, int *SnprfOverflow, char **output, char *end);
static void dopr_outch(int c, int *SnprfOverflow, char **output, char *end);



static void
dopr(char *buffer, const char *format, va_list args, char *end, int *SnprfOverflow)
{
	int			ch;
	long_long	value;
	double		fvalue;
	int			longlongflag = 0;
	int			longflag = 0;
	int			pointflag = 0;
	int			maxwidth = 0;
	char	   *strvalue;
	int			ljust;
	int			len;
	int			zpad;
        char *output;

	output = buffer;
	while ((ch = *format++))
	{
		switch (ch)
		{
			case '%':
				ljust = len = zpad = maxwidth = 0;
				longflag = longlongflag = pointflag = 0;
		nextch:
				ch = *format++;
				switch (ch)
				{
					case 0:
						dostr("**end of format**", 0, SnprfOverflow, &output, end);
						*output = '\0';
						return;
					case '-':
						ljust = 1;
						goto nextch;
					case '0':	/* set zero padding if len not set */
						if (len == 0 && !pointflag)
							zpad = '0';
					case '1':
					case '2':
					case '3':
					case '4':
					case '5':
					case '6':
					case '7':
					case '8':
					case '9':
						if (pointflag)
							maxwidth = maxwidth * 10 + ch - '0';
						else
							len = len * 10 + ch - '0';
						goto nextch;
					case '*':
						if (pointflag)
							maxwidth = va_arg(args, int);
						else
							len = va_arg(args, int);
						goto nextch;
					case '.':
						pointflag = 1;
						goto nextch;
					case 'l':
						if (longflag)
							longlongflag = 1;
						else
							longflag = 1;
						goto nextch;
					case 'u':
					case 'U':
						/* fmtnum(value,base,dosign,ljust,len,zpad) */
						if (longflag)
						{
							if (longlongflag)
								value = va_arg(args, ulong_long);
							else
								value = va_arg(args, unsigned long);
						}
						else
							value = va_arg(args, unsigned int);
						fmtnum(value, 10, 0, ljust, len, zpad, SnprfOverflow, &output, end);
						break;
					case 'o':
					case 'O':
						/* fmtnum(value,base,dosign,ljust,len,zpad) */
						if (longflag)
						{
							if (longlongflag)
								value = va_arg(args, ulong_long);
							else
								value = va_arg(args, unsigned long);
						}
						else
							value = va_arg(args, unsigned int);
						fmtnum(value, 8, 0, ljust, len, zpad, SnprfOverflow, &output, end);
						break;
					case 'i':
					case 'd':
					case 'D':
						if (longflag)
						{
							if (longlongflag)
								value = va_arg(args, long_long);
							else
								value = va_arg(args, long);
						}
						else
							value = va_arg(args, int);
						fmtnum(value, 10, 1, ljust, len, zpad, SnprfOverflow, &output, end);
						break;
					case 'x':
						if (longflag)
						{
							if (longlongflag)
								value = va_arg(args, ulong_long);
							else
								value = va_arg(args, unsigned long);
						}
						else
							value = va_arg(args, unsigned int);
						fmtnum(value, 16, 0, ljust, len, zpad, SnprfOverflow, &output, end);
						break;
					case 'X':
						if (longflag)
						{
							if (longlongflag)
								value = va_arg(args, ulong_long);
							else
								value = va_arg(args, unsigned long);
						}
						else
							value = va_arg(args, unsigned int);
						fmtnum(value, -16, 0, ljust, len, zpad, SnprfOverflow, &output, end);
						break;
					case 's':
						strvalue = va_arg(args, char *);
						if (maxwidth > 0 || !pointflag)
						{
							if (pointflag && len > maxwidth)
								len = maxwidth; /* Adjust padding */
							fmtstr(strvalue, ljust, len, zpad, maxwidth, SnprfOverflow, &output, end);
						}
						break;
					case 'c':
						ch = va_arg(args, int);
						dopr_outch(ch, SnprfOverflow, &output, end);
						break;
					case 'e':
					case 'E':
					case 'f':
					case 'g':
					case 'G':
						fvalue = va_arg(args, double);
						fmtfloat(fvalue, ch, ljust, len, maxwidth, pointflag, SnprfOverflow, &output, end);
						break;
					case '%':
						dopr_outch(ch, SnprfOverflow, &output, end);
						continue;
					default:
						dostr("???????", 0, SnprfOverflow, &output, end);
				}
				break;
			default:
				dopr_outch(ch, SnprfOverflow, &output, end);
				break;
		}
	}
	*output = '\0';
}

static void
fmtstr(char *value, int ljust, int len, int zpad, int maxwidth, int *SnprfOverflow, char **output, char *end)
{
	int			padlen,
				strlen;			/* amount to pad */

	if (value == 0)
		value = "<NULL>";
	for (strlen = 0; value[strlen]; ++strlen);	/* strlen */
	if (strlen > maxwidth && maxwidth)
		strlen = maxwidth;
	padlen = len - strlen;
	if (padlen < 0)
		padlen = 0;
	if (ljust)
		padlen = -padlen;
	while (padlen > 0)
	{
		dopr_outch(' ', SnprfOverflow, output, end);
		--padlen;
	}
	dostr(value, maxwidth, SnprfOverflow, output, end);
	while (padlen < 0)
	{
		dopr_outch(' ', SnprfOverflow, output, end);
		++padlen;
	}
}

static void
fmtnum(long_long value, int base, int dosign, int ljust, int len, int zpad, int *SnprfOverflow, char **output, char *end)
{
	int			signvalue = 0;
	ulong_long	uvalue;
	char		convert[64];
	int			place = 0;
	int			padlen = 0;		/* amount to pad */
	int			caps = 0;

	/*
	 * DEBUGP(("value 0x%x, base %d, dosign %d, ljust %d, len %d, zpad
	 * %d\n", value, base, dosign, ljust, len, zpad ));
	 */
	uvalue = value;
	if (dosign)
	{
		if (value < 0)
		{
			signvalue = '-';
			uvalue = -value;
		}
	}
	if (base < 0)
	{
		caps = 1;
		base = -base;
	}
	do
	{
		convert[place++] = (caps ? "0123456789ABCDEF" : "0123456789abcdef")
			[uvalue % (unsigned) base];
		uvalue = (uvalue / (unsigned) base);
	} while (uvalue);
	convert[place] = 0;

	if (len < 0)
	{
		/* this could happen with a "*" width spec */
		ljust = 1;
		len = -len;
	}
	padlen = len - place;
	if (padlen < 0)
		padlen = 0;
	if (ljust)
		padlen = -padlen;

	/*
	 * DEBUGP(( "str '%s', place %d, sign %c, padlen %d\n",
	 * convert,place,signvalue,padlen));
	 */
	if (zpad && padlen > 0)
	{
		if (signvalue)
		{
			dopr_outch(signvalue, SnprfOverflow, output, end);
			--padlen;
			signvalue = 0;
		}
		while (padlen > 0)
		{
			dopr_outch(zpad, SnprfOverflow, output, end);
			--padlen;
		}
	}
	while (padlen > 0)
	{
		dopr_outch(' ', SnprfOverflow, output, end);
		--padlen;
	}
	if (signvalue)
		dopr_outch(signvalue, SnprfOverflow, output, end);
	while (place > 0)
		dopr_outch(convert[--place], SnprfOverflow, output, end);
	while (padlen < 0)
	{
		dopr_outch(' ', SnprfOverflow, output, end);
		++padlen;
	}
}

static void
fmtfloat(double value, char type, int ljust, int len, int precision, int pointflag, int *SnprfOverflow, char **output, char *end)
{
	char		fmt[32];
	char		convert[512];
	int			padlen = 0;		/* amount to pad */

	/* we rely on regular C library's sprintf to do the basic conversion */
	if (pointflag)
		sprintf(fmt, "%%.%d%c", precision, type);
	else
		sprintf(fmt, "%%%c", type);
	sprintf(convert, fmt, value);

	if (len < 0)
	{
		/* this could happen with a "*" width spec */
		ljust = 1;
		len = -len;
	}
	padlen = len - strlen(convert);
	if (padlen < 0)
		padlen = 0;
	if (ljust)
		padlen = -padlen;

	while (padlen > 0)
	{
		dopr_outch(' ', SnprfOverflow, output, end);
		--padlen;
	}
	dostr(convert, 0, SnprfOverflow, output, end);
	while (padlen < 0)
	{
		dopr_outch(' ', SnprfOverflow, output, end);
		++padlen;
	}
}

static void
dostr(char *str, int cut, int *SnprfOverflow, char **output, char *end)
{
	if (cut)
	{
		while (*str && cut-- > 0)
			dopr_outch(*str++, SnprfOverflow, output, end);
	}
	else
	{
		while (*str)
			dopr_outch(*str++, SnprfOverflow, output, end);
	}
}

static void
dopr_outch(int c, int *SnprfOverflow, char **output, char *end)
{

#ifdef NOT_USED
	if (iscntrl((unsigned char) c) && c != '\n' && c != '\t')
	{
		c = '@' + (c & 0x1F);
		if (end == 0 || *output < end)
			*output++ = '^';
	}
#endif
	if (end == 0 || *output < end) {
          *output[0] = c;
	  *output=*output+1;
	} else {
	  (*SnprfOverflow)++;
	}
}

