/* MCVE v3.x C API
   (c) 2002 by Main Street Softworks, Inc.
   Written by Brad House

   This API is being released to the public domain to be modified and used in any manor the holder of this code sees fit.
   For any questions please contact  support@mainstreetsoftworks.com
*/

#include "libmcve_main.h"
#include "mcve.h"

long MCVE_FindQueueNumber(MCVE_CONN *myconn, char *identifier)
{
  long i;
  _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];

  for (i=0; i<conn->queue_length; i++) {
    if (conn->queue[i].status == MCVE_PENDING)
      if (strcmp(conn->queue[i].identifier, identifier) == 0)
         return(i);
  }
  return(-1);
}

void MCVE_StoreResponseValues(MCVE_CONN *myconn, long queuenum, char *option, char *value)
{
  _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
  if (strcmp(option, "code") == 0) {
    if (strcmp(value, "AUTH") == 0)
      conn->queue[queuenum].code=MCVE_AUTH;
    else if (strcmp(value, "DENY") == 0)
      conn->queue[queuenum].code=MCVE_DENY;
    else if (strcmp(value, "DUPL") == 0)
      conn->queue[queuenum].code=MCVE_DUPL;
    else if (strcmp(value, "CALL") == 0)
      conn->queue[queuenum].code=MCVE_CALL;
    else if (strcmp(value, "PKUP") == 0)
      conn->queue[queuenum].code=MCVE_PKUP;
    else if (strcmp(value, "RETRY") == 0)
      conn->queue[queuenum].code=MCVE_RETRY;
    else if (strcmp(value, "SETUP") == 0)
      conn->queue[queuenum].code=MCVE_SETUP;
    else if (strcmp(value, "SUCCESS") == 0)
      conn->queue[queuenum].code=MCVE_SUCCESS;
    else if (strcmp(value, "FAIL") == 0)
      conn->queue[queuenum].code=MCVE_FAIL;
    else if (strcmp(value, "TIMEOUT") == 0)
      conn->queue[queuenum].code=MCVE_TIMEOUT;
   else
      conn->queue[queuenum].code=MCVE_DENY;
  } else if (strcmp(option, "avs") == 0) {
    if (strcmp(value, "GOOD") == 0)
      conn->queue[queuenum].avs=MCVE_GOOD;
    else if (strcmp(value, "BAD") == 0)
      conn->queue[queuenum].avs=MCVE_BAD;
    else if (strcmp(value, "STREET") == 0)
      conn->queue[queuenum].avs=MCVE_STREET;
    else if (strcmp(value, "ZIP") == 0)
      conn->queue[queuenum].avs=MCVE_ZIP;
    else
      conn->queue[queuenum].avs=MCVE_UNKNOWN;
  } else if (strcmp(option, "cv") == 0) {
    if (strcmp(value, "GOOD") == 0)
      conn->queue[queuenum].cv=MCVE_GOOD;
    else if (strcmp(value, "BAD") == 0)
      conn->queue[queuenum].cv=MCVE_BAD;
    else
      conn->queue[queuenum].cv=MCVE_UNKNOWN;
  } else if (strcmp(option, "auth") == 0) {
    conn->queue[queuenum].auth=strdup(value);
  } else if (strcmp(option, "verbiage") == 0) {
    conn->queue[queuenum].text=strdup(value);
  } else if (strcmp(option, "item") == 0) {
    conn->queue[queuenum].item=strdup(value);
  } else if (strcmp(option, "batch") == 0) {
    conn->queue[queuenum].batch=strdup(value);
  } else if (strcmp(option, "ttid") == 0 || strcmp(option, "tid") == 0 || strcmp(option, "sid") == 0 || strcmp(option, "did") == 0) {
    conn->queue[queuenum].tid=atoll(value);
  }
}

void MCVE_ProcessResponse(MCVE_CONN *myconn, char *identifier, char *message)
{
  _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
  long num;
  long pos=0;
  char *line=NULL, *option=NULL, *value=NULL;
  int is_comma_delimiated=0;

  MCVE_lock(myconn, MCVE_LOCK_QUEUE);
  num=MCVE_FindQueueNumber(myconn, identifier);
  if (num == -1) {
    MCVE_unlock(myconn, MCVE_LOCK_QUEUE);
    return;
  }
  conn->queue[num].status=MCVE_DONE;

  while ((pos=MCVE_ReadLine(message, pos, &line)) != -1) {
     if (!MCVE_ParseLine(line, &option, &value)) {
       is_comma_delimiated=1;
       conn->queue[num].iscommadelimiated=1;
       free(line);
       break;
     }
     MCVE_StoreResponseValues(myconn, num, option, value);
     conn->queue[num].parsed_resp=(MCVE_TRANS *)realloc(conn->queue[num].parsed_resp, (conn->queue[num].resp_fields+1)*sizeof(MCVE_TRANS));
     conn->queue[num].parsed_resp[conn->queue[num].resp_fields].key=MC_SAFE_strdup(option);
     conn->queue[num].parsed_resp[conn->queue[num].resp_fields].value=MC_SAFE_strdup(value);
     conn->queue[num].resp_fields++;
     free(option);
     free(value);
     free(line);
  }
  if (is_comma_delimiated) {
    conn->queue[num].response=(char *)malloc(strlen(message)+1);
    strcpy(conn->queue[num].response, message);
    conn->queue[num].code=MCVE_SUCCESS;
  }
  MCVE_unlock(myconn, MCVE_LOCK_QUEUE);
}

void MCVE_ProcessBuffer(MCVE_CONN *myconn)
{
  _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
  int cutoff=0, done=0;
  char *temp=NULL, *temp2=NULL;
  char *response=NULL, *identifier=NULL, *message=NULL;

  MCVE_lock(myconn, MCVE_LOCK_IN);
  while (!done) {
    if (conn->inbuf != NULL) {
   //   printf("Checking for a response\r\n");
      cutoff=MCVE_IP_Get_Response(conn->inbuf, &response);
      if (cutoff != -1) {
        // Separate out the identifier and message
    //    printf("Got a full response!\r\n");
        if (MCVE_IP_Get_Response_Message(response, &message) && MCVE_IP_Get_Response_Identifier(response, &identifier)) {
           MCVE_ProcessResponse(myconn, identifier, message);
        }
        if (message != NULL) {  free(message); }
        if (identifier != NULL) {  free(identifier); }
        free(response);

        // --------------CUT TRANSACTION OUT OF INBUF ------------------
        temp=conn->inbuf+cutoff;
        if (strlen(temp) > 0) {
          temp2=strdup(temp);
          temp=NULL;
          conn->inbuf=(char *)realloc(conn->inbuf, strlen(temp2)+1);
          memset(conn->inbuf, 0, strlen(temp2)+1);
          strcpy(conn->inbuf, temp2);
	  free(temp2);
        } else {
          free(conn->inbuf);
          conn->inbuf=NULL;
        }
        // -------------END CUTTING TRANSACTION OUT OF INBUF--------
      } else
        done=1;
    } else
      done=1;
  }
  MCVE_unlock(myconn, MCVE_LOCK_IN);
}




long MCVE_OpenQueueSlot(MCVE_CONN *myconn)
{
   _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
   long i;
   long pos=-1;

   for (i=0; i<conn->queue_length; i++) {
     if (conn->queue[i].status == MCVE_UNUSED) {
        pos=i;
        break;
     }
   }
   if (pos == -1) {
     if (conn->queue == NULL) {
        conn->queue = (MCVE_QUEUE *)malloc(1 * sizeof(MCVE_QUEUE));
        pos=0;
     } else {
       conn->queue = (MCVE_QUEUE *)realloc(conn->queue, (conn->queue_length+1)*sizeof(MCVE_QUEUE));
       pos=conn->queue_length;
     }
     memset(conn->queue[pos].identifier, 0, 30);
     conn->queue[pos].status = MCVE_UNUSED;
     conn->queue_length++;
   }
   return(pos);
}



char *MCVE_StructureTransaction(MCVE_CONN *myconn, long identifier)
{
  char *transaction=NULL;
  char temp[1024];
  int i;
  _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];

  transaction=(char *)malloc(4096);
  memset(transaction, 0, 4096);

  if (conn->queue[identifier].type == MC_TRAN_PING) {
    strcpy(transaction, "PING\r\n");
    return(transaction);
  }
  for (i=0; i<conn->queue[identifier].transaction_fields; i++) {
    MCVE_snprintf(temp, 1024, "%s=%s\r\n", conn->queue[identifier].transaction[i].key, conn->queue[identifier].transaction[i].value);
    strcat(transaction, temp);
  }
  return(transaction);
}

int MCVE_ClearQueueSlot(MCVE_CONN *myconn, long pos)
{
  _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];

  conn->queue[pos].type=0;
  conn->queue[pos].admin=0;
  conn->queue[pos].auth=NULL;
  conn->queue[pos].text=NULL;
  conn->queue[pos].item=NULL;
  conn->queue[pos].batch=NULL;
  conn->queue[pos].code=-1;
  conn->queue[pos].tid=-1;
  conn->queue[pos].resp_fields=0;
  conn->queue[pos].parsed_resp=NULL;
  conn->queue[pos].response=NULL;
  conn->queue[pos].separated=NULL;
  conn->queue[pos].transaction_fields=0;
  conn->queue[pos].transaction=NULL;
  return(1);
}

/*
long MCVE_AddOutstandingAuth(MCVE_CONN *myconn, char *identifier)
{
  _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
  long pos;
  MCVE_lock(myconn, MCVE_LOCK_QUEUE);
  pos=MCVE_OpenQueueSlot(myconn);
  strncpy(conn->queue[pos].identifier, identifier, 29);
  conn->queue[pos].status=MCVE_PENDING;
  conn->queue[pos].auth=NULL;
  conn->queue[pos].text=NULL;
  conn->queue[pos].item=NULL;
  conn->queue[pos].batch=NULL;
  conn->queue[pos].code=-1;
  conn->queue[pos].tid=-1;
  conn->queue[pos].response=NULL;
  conn->queue[pos].separated=NULL;
  conn->outstanding_auths++;
  MCVE_unlock(myconn, MCVE_LOCK_QUEUE);
  return(pos);
}
*/

int MCVE_SendTransaction(MCVE_CONN *myconn, long identifier)
{
  _MCVE_CONN *conn=(_MCVE_CONN *)myconn[0];
  char *transaction=NULL;
  char *id=NULL;
  int ret=0;

  id=MCVE_GenerateIdentifier();
  transaction=MCVE_StructureTransaction(myconn, identifier);

  if (conn->conn_method == MCVE_FILE)
    ret=MCVE_SendTransaction_File(myconn, id, transaction);
#ifndef DISABLE_IP
  else
    ret=MCVE_SendTransaction_IP(myconn, id, transaction);
#endif

  free(transaction);

  //pos=MCVE_AddOutstandingAuth(myconn, identifier);
  MCVE_lock(myconn, MCVE_LOCK_QUEUE);
  conn->queue[identifier].status=MCVE_PENDING;
  strcpy(conn->queue[identifier].identifier, id);
  free(id);
  MCVE_unlock(myconn, MCVE_LOCK_QUEUE);

  return(ret);
}
